from mimesis.providers.base import BaseProvider


class ClothingSizes(BaseProvider):
    """Class for generate clothing sizes data"""

    def international_size(self) -> str:
        """Get a random size in international format.

        :return: Clothing size.

        :Example:
            XXL.
        """
        sizes = (
            'L',
            'M',
            'S',
            'XL',
            'XS',
            'XXL',
            'XXS',
            'XXXL',
        )

        return self.random.choice(sizes)

    def european_size(self) -> int:
        """Generate a random clothing size in European format.

        :return: Clothing size.

        :Example:
            42
        """
        sizes = [i for i in range(40, 62) if i % 2 == 0]
        return self.random.choice(sizes)

    def custom_size(self, minimum: int = 40, maximum: int = 62) -> int:
        """Generate clothing size using custom format.

        :param int minimum: Minimum value.
        :param int maximum: Maximum value.
        :return: Clothing size.

        :Example:
            44
        """
        return self.random.randint(minimum, maximum)



import hashlib
import uuid
from binascii import hexlify
from typing import Optional

from mimesis.enums import Algorithm
from mimesis.providers.base import BaseProvider
from mimesis.providers.text import Text
from mimesis.typing import Bytes


class Cryptographic(BaseProvider):
    """This class provides support cryptographic data.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__words = Text('en')._data['words']

    def uuid(self) -> str:
        """Generate random UUID.

        :return: UUID
        """
        bits = self.random.getrandbits(128)
        return str(uuid.UUID(int=bits))

    def hash(self, algorithm: Optional[Algorithm] = None) -> str:
        """Generate random hash.

        :param algorithm: Enum object Algorithm.
        :return: Hash.
        :raises NonEnumerableError: if algorithm is not supported.
        """
        key = self._validate_enum(
            item=algorithm,
            enum=Algorithm,
        )

        if hasattr(hashlib, key):
            fn = getattr(hashlib, key)
            _hash = fn(self.uuid().encode())
            return _hash.hexdigest()

    def bytes(self, entropy: int = 32) -> Bytes:
        """Get a random byte string containing *entropy* bytes.

        The string has *entropy* random bytes, each byte converted to two
        hex digits.

        :param entropy: Number of bytes.
        :return: Bytes.
        :rtype: bytes
        """
        return self.random.urandom(entropy)

    def token(self, entropy: int = 32) -> str:
        """Return a random text string, in hexadecimal.

        :param entropy: Number of bytes.
        :return: Token.
        """
        token = hexlify(self.bytes(entropy))
        return token.decode('ascii')

    @staticmethod
    def salt() -> str:
        """Generate salt (not cryptographically safe) using uuid4().

        :return: Salt.
        """
        return uuid.uuid4().hex

    def mnemonic_code(self, length: int = 12) -> str:
        """Generate pseudo mnemonic code.

        :param length: Length of code (number of words).
        :return: Mnemonic code.
        """
        words = self.__words['normal']
        code = [self.random.choice(words) for _ in range(length)]
        return ' '.join(code)



from typing import Any, Optional

from mimesis.exceptions import NonEnumerableError
from mimesis.helpers import Random
from mimesis.utils import locale_info, setup_locale


class ValidateEnumMixin(object):
    """
    A mixin which helps validate enums.
    """

    @staticmethod
    def _validate_enum(item: Any, enum: Any) -> Any:
        """Validate enum parameter of method in subclasses of BaseProvider.

        :param item: Item of enum object.
        :param enum: Enum object.
        :return: Value of item.
        :raises NonEnumerableError: if ``item`` not in ``enum``.
        """

        if item is None:
            result = enum.get_random_item()
        elif item and isinstance(item, enum):
            result = item
        else:
            raise NonEnumerableError(enum)

        return result.value


class StrMixin(object):
    """
    A mixin for showing information about the current
    locale of the current data provider.
    """

    def __str__(self) -> str:
        if hasattr(self, 'locale'):
            locale = getattr(self, 'locale')
            return '{}:{}:{}'.format(
                self.__class__.__name__,
                locale,
                locale_info(locale),
            )
        return '{}'.format(
            self.__class__.__name__)


class BaseProvider(ValidateEnumMixin, StrMixin):
    """
    This is a base class for all data providers.
    """

    def __init__(self, locale: Optional[str] = None,
                 seed: Optional[int] = None) -> None:
        """Base constructor for all providers.

        :param str locale: Current locale. Default is 'en'.
        :param int seed: Seed to all the random functions. Default is 'None'.
        """
        self.seed = seed
        self.random = Random()
        self.locale = setup_locale(locale)

        if seed is not None:
            self.random.seed(self.seed)

    def get_current_locale(self) -> str:
        """Current locale of provider.

        ..Note: Default for all providers is locale ``en``.
        """
        return self.locale



from mimesis.data import (CRYPTOCURRENCY_ISO_CODES, CRYPTOCURRENCY_SYMBOLS,
                          CURRENCY_ISO_CODES, CURRENCY_SYMBOLS)
from mimesis.providers.base import BaseProvider
from mimesis.utils import pull


class Business(BaseProvider):
    """Class for generating data for business."""

    def __init__(self, *args, **kwargs):
        """
        :param str locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self._data = pull('business.json', self.locale)

    def company_type(self, abbr: bool = False) -> str:
        """Get a random type of business entity.

        :param bool abbr: Abbreviated company type.
        :return: Types of business entity.

        :Example:
            Incorporated.
        """
        key = 'abbr' if abbr else 'title'
        company_type = self._data['company'].get(
            'type').get(key)
        return self.random.choice(company_type)

    def company(self) -> str:
        """Get a random company name.

        :return: Company name.

        :Example:
            Gamma Systems.
        """
        companies = self._data['company'].get('name')
        return self.random.choice(companies)

    def copyright(self) -> str:
        """Generate a random copyright.

        :return: Dummy copyright of company.

        :Example:
            © 1990-2016 Komercia, Inc.
        """
        return '© {company}, {company_type}'.format(
            company=self.company(),
            company_type=self.company_type(
                abbr=True,
            ),
        )

    def currency_iso_code(self, crypto: bool = False) -> str:
        """Get a currency code. ISO 4217 format.

        :param crypto: Return ISO code of cryptocurrency.
        :return: Currency code.

        :Example:
            RUR.
        """
        if crypto:
            codes = CRYPTOCURRENCY_ISO_CODES
        else:
            codes = CURRENCY_ISO_CODES

        return self.random.choice(codes)

    def currency_symbol(self, crypto: bool = False):
        """Get a currency symbol for current locale.

        :param crypto: Return symbol of cryptocurrency.
        :return: Currency symbol.

        :Example:
            Ƀ
        """
        if crypto:
            return self.random.choice(CRYPTOCURRENCY_SYMBOLS)
        else:
            return CURRENCY_SYMBOLS[self.locale]

    def price(self, minimum: float = 10.00,
              maximum: float = 1000.00) -> str:
        """Generate a random price.

        :param float minimum: Max value of price.
        :param float maximum: Min value of price.
        :return: Price.

        :Example:
            599.99 $.
        """
        currencies = CURRENCY_SYMBOLS
        price = self.random.uniform(minimum, maximum)

        fmt = '{0:.2f} {1}'

        if self.locale in currencies:
            return fmt.format(price, currencies[self.locale])

        return fmt.format(price, currencies['default'])

    def price_in_btc(self, minimum: float = 0, maximum: float = 2) -> str:
        """Generate random price in BTC.

        :param minimum: Minimum value of price
        :param maximum: Maximum value of price.
        :return: Price in BTC.

        :Example:
            0.5885238 BTC
        """
        price = self.random.uniform(minimum, maximum)
        return '{:.7f} BTC'.format(price)



from mimesis.data import (CPU, CPU_CODENAMES, GENERATION, GENERATION_ABBR,
                          GRAPHICS, HDD_SSD, MANUFACTURERS, PHONE_MODELS,
                          RESOLUTIONS, SCREEN_SIZES)
from mimesis.providers.base import BaseProvider


class Hardware(BaseProvider):
    """Class for generate data about hardware."""

    def resolution(self) -> str:
        """Get a random screen resolution.

        :return: Resolution of screen.

        :Example:
            1280x720.
        """
        return self.random.choice(RESOLUTIONS)

    def screen_size(self) -> str:
        """Get a random size of screen in inch.

        :return: Screen size.

        :Example:
            13″.
        """
        return self.random.choice(SCREEN_SIZES)

    def cpu(self) -> str:
        """Get a random CPU name.

        :return: CPU name.

        :Example:
            Intel® Core i7.
        """
        return self.random.choice(CPU)

    def cpu_frequency(self) -> str:
        """Get a random frequency of CPU.

        :return: Frequency of CPU.

        :Example:
            4.0 GHz.
        """
        cf = self.random.uniform(1.5, 4.3)
        return '{0:.1f}GHz'.format(cf)

    def generation(self, abbr: bool = False) -> str:
        """Get a random generation.

        :return: Generation of something.

        :Example:
             6th Generation.
        """
        if not abbr:
            return self.random.choice(GENERATION)

        return self.random.choice(GENERATION_ABBR)

    def cpu_codename(self) -> str:
        """Get a random CPU code name.

        :return: CPU code name.

        :Example:
            Cannonlake.
        """
        return self.random.choice(CPU_CODENAMES)

    def ram_type(self) -> str:
        """Get a random RAM type.

        :return: Type of RAM.

        :Example:
            DDR3.
        """
        ram_types = ('DDR2', 'DDR3', 'DDR4')
        return self.random.choice(ram_types)

    def ram_size(self) -> str:
        """Get a random size of RAM.

        :return: RAM size.

        :Example:
            16GB.
        """
        sizes = ('4', '6', '8', '16', '32', '64')
        return self.random.choice(sizes) + 'GB'

    def ssd_or_hdd(self) -> str:
        """Get a random value from list.

        :return: HDD or SSD.

        :Example:
            512GB SSD.
        """
        return self.random.choice(HDD_SSD)

    def graphics(self) -> str:
        """Get a random graphics.

        :return: Graphics.

        :Example:
            Intel® Iris™ Pro Graphics 6200.
        """
        return self.random.choice(GRAPHICS)

    def manufacturer(self) -> str:
        """Get a random manufacturer.

        :return: Manufacturer.

        :Example:
            Dell.
        """
        return self.random.choice(MANUFACTURERS)

    def phone_model(self) -> str:
        """Get a random phone model.

        :return: Phone model.

        :Example:
            Nokia Lumia 920.
        """
        return self.random.choice(PHONE_MODELS)



from mimesis.builtins.base import BaseSpecProvider
from mimesis.utils import pull


class GermanySpecProvider(BaseSpecProvider):
    """
    Specific-provider of misc data for Deutschland.
    """

    def __init__(self):
        super().__init__()
        self._data = pull('builtin.json', 'de')

    class Meta:
        name = 'germany_provider'

    def noun(self, plural: bool = False) -> str:
        """Return a random noun in German.

        :param bool plural: Return noun in plural.
        :return: Noun.
        """

        key = 'plural' if \
            plural else 'noun'

        return self.random.choice(self._data[key])



from mimesis.providers.base import BaseProvider
from mimesis.utils import pull


class Food(BaseProvider):
    """Class for Food, i.e fruits, vegetables, berries and other."""

    def __init__(self, *args, **kwargs):
        """
        :param str locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self._data = pull('food.json', self.locale)

    def vegetable(self) -> str:
        """Get a random vegetable.

        :return: Vegetable name.

        :Example:
            Tomato.
        """
        vegetables = self._data['vegetables']
        return self.random.choice(vegetables)

    def fruit(self) -> str:
        """Get a random name of fruit or berry .

        :return: Fruit name.

        :Example:
            Banana.
        """
        fruits = self._data['fruits']
        return self.random.choice(fruits)

    def dish(self) -> str:
        """Get a random dish for current locale.

        :return: Dish name.

        :Example:
            Ratatouille.
        """
        dishes = self._data['dishes']
        return self.random.choice(dishes)

    def spices(self) -> str:
        """Get a random spices or herbs.

        :return: Spices or herbs.

        :Example:
            Anise.
        """
        spices = self._data['spices']
        return self.random.choice(spices)

    def drink(self) -> str:
        """Get a random drink.

        :return: Alcoholic drink.

        :Example:
            Vodka.
        """
        drinks = self._data['drinks']
        return self.random.choice(drinks)



from typing import Any, Optional


class UnsupportedAlgorithm(AttributeError):
    """ Raised when the user wants to use an unsupported algorithm. """


class UnsupportedLocale(KeyError):
    """ Raised when a locale isn't supported. """

    def __init__(self, locale: Optional[str] = None) -> None:
        self.locale = locale
        self.message = 'Locale «{}» is not supported'

    def __str__(self) -> str:
        return self.message.format(self.locale)


class UndefinedSchema(ValueError):
    """ Raised when schema is empty. """

    def __str__(self) -> str:
        return 'Schema should be defined in lambda.'


class NonEnumerableError(TypeError):
    """ Raised when object is not instance of Enum """

    message = 'You should use one item of: «{}» of the object mimesis.enums.{}'

    def __init__(self, enum_obj: Any) -> None:
        if enum_obj:
            self.name = enum_obj
            self.items = ', '.join([str(i) for i in enum_obj])
        else:
            self.items = ''

    def __str__(self) -> str:
        return self.message.format(self.items,
                                   self.name.__name__)


class UnsupportedField(ValueError):
    def __init__(self, name: Optional[str] = None) -> None:
        self.name = name
        self.message = 'Field «{}» is not supported.'

    def __str__(self) -> str:
        return self.message.format(self.name)


class UndefinedField(ValueError):
    def __str__(self) -> str:
        return 'Undefined field. Filed cannot be None.'



import os
import random
from typing import Any, List, Union

__all__ = ['Random']


class Random(random.Random):
    """Custom Random() class for the possibility of extending."""

    def randints(self, amount: int = 3,
                 a: int = 1, b: int = 100) -> List[int]:
        """Generate list of random integers.

        :param int amount: Amount of elements.
        :param int a: Minimum value of range.
        :param int b: Maximum value of range.
        :return: List of random integers.
        :rtype: list
        :raises ValueError: if amount less or equal to zero.
        """

        if amount <= 0:
            raise ValueError('Amount out of range.')

        return [self.randint(a, b)
                for _ in range(amount)]

    @staticmethod
    def urandom(*args: Any, **kwargs: Any) -> bytes:
        """Return a bytes object containing random bytes

        :return: Bytes.
        :rtype: bytes
        """
        return os.urandom(*args, **kwargs)

    def schoice(self, seq: Union[tuple, list], end: int = 10) -> str:
        """Choice function which returns string created from sequence.

        :param seq: Sequence of letters or digits.
        :type seq: tuple or list
        :param int end: Max value.
        :return: Single string.
        """
        seq = [self.choice(seq) for _ in range(end)]
        return ''.join(seq)

    def custom_code(self, mask: str = '@###',
                    char: str = '@', digit: str = '#') -> str:
        """Generate custom code using ascii uppercase and random integers.
        :param mask: Mask of code.
        :param char: Placeholder for characters.
        :param digit: Placeholder for digits.
        :return: Custom code.
        :Example:
            5673-AGFR-SFSFF-1423-4/AD.
        """
        char_code = ord(char)
        digit_code = ord(digit)
        code = bytearray(len(mask))

        def random_int(a: int, b: int) -> int:
            b = b - a
            return int(self.random() * b) + a

        _mask = mask.encode()
        for i, p in enumerate(_mask):
            if p == char_code:
                a = random_int(65, 91)  # A-Z
            elif p == digit_code:
                a = random_int(48, 58)  # 0-9
            else:
                a = p
            code[i] = a
        return code.decode()



from typing import Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.enums import Gender
from mimesis.utils import pull


class UkraineSpecProvider(BaseSpecProvider):
    def __init__(self):
        super().__init__()
        self._data = pull('builtin.json', 'uk')

    class Meta:
        name = 'ukraine_provider'

    def patronymic(self, gender: Optional[Gender] = None) -> str:
        """Generate random patronymic name.

        :param gender: Gender of person.
        :type gender: str or int
        :return: Patronymic name.
        """
        gender = self._validate_enum(gender, Gender)
        patronymics = self._data['patronymic'][gender]
        return self.random.choice(patronymics)



from enum import Enum, EnumMeta
from random import choice
from typing import Any


class AllowRandom(EnumMeta):
    """This metaclass allows method ``get_random_item()`` which
    returns random field of enum.
    """

    def get_random_item(self) -> Any:
        return choice(list(self))


class PortRange(Enum):
    """Value for a method ``Internet.port()`` which
    takes parameter``port_range``.

    Example:

    >>> internet.port(port_range=PortRange.REGISTERED)
    '8080'
    """
    ALL = (1, 65535)
    WELL_KNOWN = (1, 1023)
    EPHEMERAL = (49152, 65535)
    REGISTERED = (1024, 49151)


class Gender(Enum, metaclass=AllowRandom):
    """Value for a lot of methods which
    takes argument ``gender``.

    Example:

    >>> personal.full_name(gender=Gender.FEMALE)
    'Jessica Larson'
    """
    FEMALE = 'female'
    MALE = 'male'


class TitleType(Enum, metaclass=AllowRandom):
    """Value for a method ``Personal.title()`` which
    takes parameter``title_type``.

    Example:

    >>> personal.title(title_type=TitleType.TYPICAL)
    'Mr.'
    """
    TYPICAL = 'typical'
    ACADEMIC = 'academic'


class CardType(Enum, metaclass=AllowRandom):
    """Value for a method ``Payment.credit_card_number()`` which
    takes parameter ``card_type``.

    Example:

    >>> payment.credit_card_number(card_type=CardType.VISA)
    '4116 9018 7744 0125'
    """
    MASTER_CARD = 'MasterCard'
    VISA = 'Visa'
    AMERICAN_EXPRESS = 'American Express'


class Algorithm(Enum, metaclass=AllowRandom):
    """Value for methods which take argument ``algorithm``.

    Example:

    >>> cryptographic.hash(algorithm=Algorithm.MD5)
    '6023658e8cc9c97edfd51cf2fabf1faf'
    """
    MD5 = 'md5'
    SHA1 = 'sha1'
    SHA224 = 'sha224'
    SHA256 = 'sha256'
    SHA384 = 'sha384'
    SHA512 = 'sha512'


class TLDType(Enum, metaclass=AllowRandom):
    """Value for a method ``Internet.top_level_domain()`` which
    takes parameter ``tld_type``.

    Example:

    >>> internet.top_level_domain(tld_type=TLDType.GEOTLD)
    '.moscow'
    """
    CCTLD = 'cctld'
    GTLD = 'gtld'
    GEOTLD = 'geotld'
    UTLD = 'utld'
    STLD = 'stld'


class Layer(Enum, metaclass=AllowRandom):
    """Value for method ``Internet.network_protocol()`` which
    takes parameter ``layer``.

    Example:

    >>> internet.network_protocol(layer=Layer.APPLICATION)
    'BitTorrent'
    """
    APPLICATION = 'application'
    DATA_LINK = 'data_link'
    NETWORK = 'network'
    PHYSICAL = 'physical'
    PRESENTATION = 'presentation'
    SESSION = 'session'
    TRANSPORT = 'transport'


class FileType(Enum, metaclass=AllowRandom):
    """Value for methods which takes parameter ``file_type``.

    Example:

    >>> file.extension(file_type=FileType.SOURCE)
    '.py'
    """
    SOURCE = 'source'
    TEXT = 'text'
    DATA = 'data'
    AUDIO = 'audio'
    VIDEO = 'video'
    IMAGE = 'image'
    EXECUTABLE = 'executable'
    COMPRESSED = 'compressed'


class MimeType(Enum, metaclass=AllowRandom):
    """Value for method ``File.mime_type()`` which
    takes parameter ``type_``.

    Example:

    >>> file.mime_type(type_=MimeType.APPLICATION)
    'application/alto-directory+json'
    """
    APPLICATION = 'application'
    AUDIO = 'audio'
    IMAGE = 'image'
    MESSAGE = 'message'
    TEXT = 'text'
    VIDEO = 'video'


class PrefixSign(Enum, metaclass=AllowRandom):
    """Value for method ``UnitSystem.prefix()`` which
    takes parameter ``sign``.

    Example:

    >>> unitsystem.prefix(sign=PrefixSign.NEGATIVE)
    'yocto'
    """
    POSITIVE = 'positive'
    NEGATIVE = 'negative'


class CountryCode(Enum, metaclass=AllowRandom):
    """Value for method ``Address.country_iso_code()`` which
    takes parameter ``fmt``.

    Example:

    >>> address.country_iso_code(fmt=CountryCode.ISO3)
    'AND'
    """
    ISO2 = 'iso2'
    ISO3 = 'iso3'
    NUMERIC = 'numeric'


class ISBNFormat(Enum, metaclass=AllowRandom):
    """Value for method ``Code.isbn()`` which
    takes parameter ``fmt``.

    Example:

    >>> code.isbn(fmt=ISBNFormat.ISBN10)
    '1-64396-236-2'
    """
    ISBN13 = 'isbn-13'
    ISBN10 = 'isbn-10'


class EANFormat(Enum, metaclass=AllowRandom):
    """Value for method ``Code.ean()`` which
    takes parameter ``fmt``.

    Example:

    >>> code.ean(fmt=EANFormat.EAN8)
    '09959590'
    """
    EAN8 = 'ean-8'
    EAN13 = 'ean-13'


class SocialNetwork(Enum, metaclass=AllowRandom):
    """Value for method ``Personal.social_media_profile()`` which
    takes parameter ``site``.

    Example:

    >>> personal.social_media_profile(site=SocialNetwork.INSTAGRAM)
    'https://www.instagram.com/cacara_1844'
    """
    FACEBOOK = 'facebook'
    TWITTER = 'twitter'
    INSTAGRAM = 'instagram'
    VK = 'vk'


class UnitName(Enum, metaclass=AllowRandom):
    """Value for method ``UnitSystem.unit()`` which
    takes parameter ``name``.

    Example:

    >>> unitsystem.unit(name=UnitName.MASS)
    'gramm'
    """
    MASS = ('gram', 'gr')
    INFORMATION = ('byte', 'b')
    THERMODYNAMIC_TEMPERATURE = ('kelvin', 'K')
    AMOUNT_OF_SUBSTANCE = ('mole', 'mol')
    ANGLE = ('radian', 'r')
    SOLID_ANGLE = ('steradian', '㏛')
    FREQUENCY = ('hertz', 'Hz')
    FORCE = ('newton', 'N')
    PRESSURE = ('pascal', 'P')
    ENERGY = ('joule', 'J')
    POWER = ('watt', 'W')
    FLUX = ('watt', 'W')
    ELECTRIC_CHARGE = ('coulomb', 'C')
    VOLTAGE = ('volt', 'V')
    ELECTRIC_CAPACITANCE = ('farad', 'F')
    ELECTRIC_RESISTANCE = ('ohm', 'Ω')
    ELECTRICAL_CONDUCTANCE = ('siemens', 'S')
    MAGNETIC_FLUX = ('weber', 'Wb')
    MAGNETIC_FLUX_DENSITY = ('tesla', 'T')
    INDUCTANCE = ('henry', 'H')
    TEMPERATURE = ('Celsius', '°C')
    RADIOACTIVITY = ('becquerel', 'Bq')



BLOOD_GROUPS = (
    'O+',
    'A+',
    'B+',
    'AB+',
    'O−',
    'A−',
    'B−',
    'AB−',
)

MUSIC_GENRE = (
    'Pop',
    'Rock',
    'Hard Rock',
    'Rhythm & Blues (R&B)',
    'Country',
    'Rock & Roll',
    'Soul',
    'Country Pop',
    'Pop Rock',
    'Heavy Metal',
    'Progressive Rock',
    'Alternative Rock',
    'Jazz',
    'New Wave',
    'Synthpop',
    'Hip Hop',
    'Folk',
    'New Age',
    'Blues Rock',
    'Ambient',
    'Ambient house',
    'Classic',
    'Neoclassic',
    'Drum and bass',
)

GENDER_SYMBOLS = (
    '♂',
    '♀',
    '⚲',
)

SEXUALITY_SYMBOLS = (
    '⚤',
    '⚢',
    '⚣',
    '⚥',
    '⚧',
    '⚪',
)

ENGLISH_LEVEL = [
    'Beginner',
    'Elementary',
    'Pre - Intermediate',
    'Intermediate',
    'Upper Intermediate',
    'Advanced',
    'Proficiency',
]

USERNAMES = [
    "ababa",
    "abalone",
    "abandoned",
    "abatis",
    "abdel",
    "abdominal",
    "abele",
    "abidal",
    "abiezer",
    "abilities",
    "abiogenesis",
    "ablution",
    "abnegative",
    "abody",
    "abolished",
    "abominable",
    "aborter",
    "aborticide",
    "abrar",
    "abrasax",
    "abridged",
    "abroad",
    "abrus",
    "abscess",
    "absconsa",
    "absit",
    "absolute",
    "absorbing",
    "abstain",
    "abstinent",
    "absurd",
    "abucco",
    "abundantia",
    "abusers",
    "acana",
    "acanthurus",
    "accidia",
    "acclimatise",
    "aceology",
    "acephalous",
    "acers",
    "achaean",
    "achamma",
    "achates",
    "achira",
    "achmetha",
    "achromic",
    "aciduria",
    "acinar",
    "ackey",
    "acree",
    "acrite",
    "acrook",
    "acryl",
    "actinon",
    "acuate",
    "acubens",
    "acute",
    "adamczyk",
    "adansonia",
    "adapa",
    "adaptive",
    "adcox",
    "addendum",
    "adders",
    "addiction",
    "addressee",
    "adebayo",
    "adena",
    "adenitis",
    "adermia",
    "adiana",
    "adine",
    "adirondack",
    "adjuster",
    "adman",
    "administrate",
    "admissible",
    "admissions",
    "admonisher",
    "adolph",
    "adolphe",
    "adoree",
    "adorer",
    "adown",
    "adriaan",
    "adriena",
    "adriene",
    "adventure",
    "adventurer",
    "adventuring",
    "adverted",
    "advertised",
    "advocate",
    "aeonian",
    "aerobatic",
    "aesthetic",
    "affiliate",
    "affluent",
    "afloat",
    "afret",
    "african",
    "afterfuture",
    "afterglow",
    "afterimage",
    "afterlife",
    "afternight",
    "against",
    "agalloch",
    "agamic",
    "agathodaemon",
    "agaty",
    "agdistis",
    "agent",
    "aggies",
    "aggregat",
    "agitprop",
    "agnail",
    "agnar",
    "agnatha",
    "agnomen",
    "agnosia",
    "agonal",
    "agosto",
    "aiding",
    "aiglet",
    "ailing",
    "aindrea",
    "airflow",
    "airish",
    "airlock",
    "airplay",
    "airtight",
    "aisling",
    "aissaoua",
    "aitch",
    "aiwan",
    "akinesia",
    "akkadian",
    "alair",
    "alaite",
    "alanine",
    "alans",
    "alanson",
    "alaric",
    "albata",
    "albian",
    "alcalde",
    "alcheringa",
    "alcindor",
    "alcoa",
    "alcove",
    "alcyone",
    "aldols",
    "aldus",
    "alegre",
    "alene",
    "alessand",
    "alexandrite",
    "alexas",
    "alexis",
    "alfred",
    "algal",
    "algedi",
    "alger",
    "algerine",
    "algonquin",
    "algorithms",
    "alidia",
    "alien",
    "aliens",
    "aligner",
    "alist",
    "alister",
    "aliyah",
    "alkalurops",
    "alkyds",
    "allan",
    "allard",
    "allay",
    "allegiances",
    "allegri",
    "alleluia",
    "allen",
    "allergy",
    "alleyway",
    "allgood",
    "allida",
    "alligator",
    "alligators",
    "alliya",
    "allowance",
    "allstate",
    "allthing",
    "alluring",
    "alluvium",
    "almadia",
    "aloed",
    "alogism",
    "alsaleh",
    "alshain",
    "alten",
    "alterant",
    "alterman",
    "altin",
    "altiplano",
    "altitudes",
    "altruistic",
    "aluminosis",
    "alumnal",
    "aluta",
    "alvarez",
    "alvaro",
    "alveolate",
    "alvera",
    "alvine",
    "alway",
    "alyda",
    "alyre",
    "amalgamation",
    "amalie",
    "amarelle",
    "amaryllis",
    "amazons",
    "ambarella",
    "amber",
    "amberoid",
    "amberous",
    "ambition",
    "ambitions",
    "ambitious",
    "ambivalent",
    "ambivert",
    "ambsace",
    "ambush",
    "ameed",
    "amelina",
    "ameliorators",
    "amgad",
    "amick",
    "amines",
    "ammar",
    "ammeter",
    "amnesia",
    "amontillado",
    "amorim",
    "amorist",
    "amorphia",
    "amour",
    "amperage",
    "amphibia",
    "amphibological",
    "amphigory",
    "amril",
    "amstutz",
    "amylan",
    "anabasis",
    "anabel",
    "anaconda",
    "anaesthetic",
    "analogue",
    "analyst",
    "analytic",
    "anamorphose",
    "ananda",
    "anaryan",
    "anasa",
    "anasazi",
    "anastasia",
    "anastasis",
    "anatolic",
    "ancestry",
    "anchors",
    "ancon",
    "andalusite",
    "ander",
    "andira",
    "andrease",
    "anemone",
    "anemony",
    "anent",
    "anesthetize",
    "angara",
    "angelate",
    "angeleyes",
    "angelika",
    "angelo",
    "anglesmith",
    "angostura",
    "anhedonia",
    "animated",
    "animations",
    "anime",
    "anisotropic",
    "anita",
    "ann-marie",
    "annale",
    "annalisa",
    "annamaria",
    "annelid",
    "annemari",
    "annie",
    "annihilates",
    "annoyer",
    "annum",
    "anoop",
    "anoraks",
    "another",
    "anstett",
    "answers",
    "antdom",
    "anteal",
    "antebellum",
    "antelope",
    "anterograde",
    "anthema",
    "anthems",
    "anthonin",
    "anthonis",
    "anthrol",
    "anthropos",
    "anthuriums",
    "antibiotic",
    "antical",
    "antically",
    "anticyclone",
    "antifowl",
    "antinome",
    "antinormal",
    "antipart",
    "antipodes",
    "antiprime",
    "antisun",
    "antivenom",
    "antivirus",
    "antonin",
    "antonym",
    "antre",
    "antsy",
    "antuan",
    "anukit",
    "anvil",
    "anything",
    "apaches",
    "apama",
    "apartness",
    "apatosaurus",
    "aperies",
    "aphelandra",
    "aphotic",
    "apina",
    "aplomb",
    "aplysia",
    "apoda",
    "apogon",
    "apolline",
    "apologies",
    "aporia",
    "apostrophe",
    "apparition",
    "appetence",
    "appetite",
    "appleblossom",
    "applejack",
    "apples",
    "appleton",
    "application",
    "appointment",
    "appraisal",
    "approachable",
    "april",
    "apron",
    "apropos",
    "aptitude",
    "apurba",
    "aqueduct",
    "arachnida",
    "arachnoid",
    "araks",
    "arana",
    "araneid",
    "arbacia",
    "arbel",
    "arcane",
    "archeus",
    "archfiend",
    "archimedes",
    "architectural",
    "architeuthis",
    "arcturia",
    "arcus",
    "ardea",
    "ardelia",
    "ardith",
    "ardys",
    "areito",
    "areopagite",
    "arethusa",
    "argento",
    "arghan",
    "argiope",
    "argonaut",
    "argument",
    "argus",
    "arias",
    "arienzo",
    "arisen",
    "arleng",
    "arles",
    "arley",
    "arlin",
    "arlyn",
    "armado",
    "armando",
    "armenoid",
    "arming",
    "armour",
    "armourer",
    "arney",
    "arnis",
    "arnone",
    "arock",
    "arora",
    "arpen",
    "arpita",
    "arracacha",
    "arranger",
    "arras",
    "arrays",
    "arrhythmia",
    "arriba",
    "arrogant",
    "arrowy",
    "arsenium",
    "arses",
    "artaba",
    "artemis",
    "arterial",
    "arteries",
    "artery",
    "arthritis",
    "article",
    "articulate",
    "artiller",
    "artiste",
    "artur",
    "arundo",
    "arvin",
    "arzun",
    "asbest",
    "asbestosis",
    "ascanius",
    "ascension",
    "ashcake",
    "ashlar",
    "ashleigh",
    "ashlie",
    "ashlin",
    "ashore",
    "ashpan",
    "ashton",
    "asians",
    "asleep",
    "asmear",
    "asoka",
    "asouth",
    "aspersion",
    "aspic",
    "aspidium",
    "aspire",
    "assailants",
    "assault",
    "assimilate",
    "assistant",
    "assuming",
    "asters",
    "astomia",
    "astonisher",
    "astor",
    "astoria",
    "astral",
    "astrolabe",
    "astronautic",
    "astrophil",
    "aswan",
    "atavistic",
    "atavus",
    "ateles",
    "athelia",
    "athene",
    "athenor",
    "athletics",
    "atique",
    "atlanta",
    "atlantis",
    "atlas",
    "atman",
    "atmospheric",
    "atoning",
    "atopy",
    "atpco",
    "attrition",
    "attune",
    "atune",
    "atwood",
    "auberge",
    "aubert",
    "aucan",
    "audette",
    "auditoria",
    "audrie",
    "augeri",
    "augurs",
    "aural",
    "aurevoir",
    "auric",
    "auricularia",
    "aurite",
    "auroras",
    "aurore",
    "australi",
    "autocall",
    "autogiro",
    "avalanche",
    "avantgarde",
    "avenger",
    "avenue",
    "averardo",
    "avernal",
    "avestan",
    "avgas",
    "aviation",
    "avigdor",
    "avouch",
    "avuncular",
    "awaken",
    "award",
    "awatch",
    "awave",
    "awest",
    "awide",
    "awoken",
    "axenic",
    "axled",
    "aymara",
    "ayoup",
    "ayukawa",
    "aywie",
    "azhar",
    "azido",
    "azine",
    "azoic",
    "aztecs",
    "azuma",
    "baalath",
    "babak",
    "babbage",
    "baboon",
    "baboonery",
    "babuina",
    "babyface",
    "baccalaureate",
    "bachelor",
    "backcross",
    "backdoor",
    "backers",
    "backgammon",
    "backpacker",
    "backspace",
    "bactrian",
    "badge",
    "badly",
    "badminton",
    "bagdad",
    "bagels",
    "bagging",
    "baggy",
    "baghdadi",
    "bagpipes",
    "bahama",
    "bahamian",
    "baited",
    "bajau",
    "bajury",
    "bakers",
    "baklavas",
    "baktun",
    "balaban",
    "balas",
    "balatron",
    "balboa",
    "balcom",
    "balcony",
    "baldest",
    "baldric",
    "ballate",
    "ballistae",
    "ballroom",
    "ballsy",
    "balmoral",
    "balmy",
    "balser",
    "balthasar",
    "balunda",
    "bambie",
    "banaba",
    "banat",
    "banda",
    "bande",
    "bandits",
    "bandmaster",
    "banes",
    "banish",
    "banky",
    "bannet",
    "banshie",
    "bantay",
    "banya",
    "baptist",
    "barbara",
    "barbi",
    "barclay",
    "bardolph",
    "bardsley",
    "bares",
    "bargeman",
    "barit",
    "barite",
    "barkery",
    "barkier",
    "barmecide",
    "barnstormer",
    "barong",
    "barrels",
    "barriere",
    "barse",
    "bartizan",
    "basco",
    "basenji",
    "bashed",
    "basile",
    "basin",
    "basis",
    "basker",
    "baskin",
    "basset",
    "bastille",
    "bastion",
    "bastioned",
    "batara",
    "batch",
    "bateau",
    "bateman",
    "bathe",
    "bathed",
    "batiks",
    "batis",
    "baton",
    "batster",
    "batten",
    "battlegrounds",
    "battlers",
    "battles",
    "battling",
    "batule",
    "batwa",
    "bauch",
    "bayard",
    "bayesian",
    "bazooka",
    "beachman",
    "beachwear",
    "bealle",
    "beasts",
    "beate",
    "beatitude",
    "beatitudes",
    "beaucoup",
    "beaulieu",
    "beaux",
    "beaverish",
    "becca",
    "beccafico",
    "bechern",
    "becke",
    "beckoning",
    "become",
    "bedding",
    "bedlamite",
    "bedroom",
    "bedrooms",
    "bedtime",
    "beebe",
    "beechy",
    "beefsteak",
    "beelol",
    "beethovenian",
    "befouled",
    "begley",
    "behalf",
    "behaviour",
    "behead",
    "beland",
    "belanda",
    "belar",
    "belay",
    "belee",
    "belich",
    "belie",
    "believe",
    "believer",
    "belight",
    "belion",
    "belissa",
    "bellboy",
    "bellhop",
    "bellmaker",
    "bellman",
    "bellwood",
    "bemar",
    "bemis",
    "bemole",
    "bemoon",
    "benares",
    "bench",
    "bender",
    "benight",
    "benjamite",
    "benjie",
    "benni",
    "benthic",
    "benthos",
    "benton",
    "benward",
    "benzein",
    "benzie",
    "beograd",
    "beppie",
    "berberine",
    "berenices",
    "beresfor",
    "berkelium",
    "berkowitz",
    "berlitz",
    "bernadette",
    "bernadin",
    "bernelle",
    "bernice",
    "berri",
    "berthe",
    "beshir",
    "beswick",
    "betaking",
    "bethel",
    "bethlehem",
    "beverlee",
    "beverly",
    "bevin",
    "bevvy",
    "bewitch",
    "bewitching",
    "beylic",
    "bezzo",
    "bhutia",
    "biabo",
    "bianka",
    "bibliomane",
    "bickford",
    "biddie",
    "bider",
    "biedermeier",
    "bifront",
    "biggers",
    "bigha",
    "bigman",
    "bigots",
    "bigwig",
    "bihai",
    "bijan",
    "bijou",
    "biker",
    "bilayer",
    "bilbo",
    "bilio",
    "billable",
    "billiards",
    "billows",
    "billowy",
    "binda",
    "binette",
    "binghi",
    "binits",
    "binman",
    "biochemical",
    "biogen",
    "biologists",
    "biomagnetic",
    "biomechanics",
    "biosynthesis",
    "biotech",
    "biotechnological",
    "biplab",
    "birdmen",
    "birdseed",
    "birendra",
    "birken",
    "birretta",
    "birthing",
    "birthmark",
    "bison",
    "biteme",
    "bitter",
    "bitterness",
    "bitton",
    "bitumen",
    "biune",
    "bizen",
    "bizonia",
    "blabber",
    "blackbelly",
    "blackberry",
    "blackbirds",
    "blackest",
    "blackfeet",
    "blackings",
    "blackleg",
    "blackman",
    "blackroot",
    "blackseed",
    "blacktongue",
    "blacky",
    "blais",
    "blame",
    "blanca",
    "blanch",
    "blasphemy",
    "blasted",
    "blaze",
    "blazed",
    "blazers",
    "blazon",
    "bleach",
    "bleaching",
    "bleeker",
    "blending",
    "blent",
    "blindeyes",
    "blinkered",
    "blissfully",
    "blisters",
    "blizz",
    "bloch",
    "blockader",
    "bloodless",
    "bloodletting",
    "bloodshedder",
    "bloodsports",
    "bloodstains",
    "blotto",
    "blowing",
    "blows",
    "blowy",
    "blubber",
    "blueberries",
    "bluebill",
    "bluecollar",
    "bluefish",
    "bluely",
    "bluestocking",
    "bluestones",
    "boarding",
    "boards",
    "boars",
    "boaster",
    "boats",
    "bobadil",
    "bobbee",
    "bobber",
    "bobbery",
    "bobby",
    "bobbye",
    "bobotie",
    "bobwhite",
    "bocoy",
    "boden",
    "bodhi",
    "bodier",
    "bogard",
    "bogert",
    "bohdan",
    "boiko",
    "boilerman",
    "boing",
    "boisset",
    "bolas",
    "boldface",
    "bolete",
    "bolio",
    "bolshevik",
    "bombsight",
    "bombyx",
    "bommer",
    "bonanzas",
    "bondon",
    "bonehead",
    "boneless",
    "bongos",
    "bonita",
    "boobies",
    "boodie",
    "boohoo",
    "booking",
    "bookmaker",
    "bookworm",
    "boons",
    "boose",
    "boosters",
    "boother",
    "bopeep",
    "borak",
    "boran",
    "borane",
    "borderless",
    "boreades",
    "borgh",
    "bornagain",
    "borneo",
    "boroughs",
    "borrelli",
    "borsht",
    "boser",
    "bosom",
    "bosomy",
    "bossa",
    "bosset",
    "bossman",
    "bottlefeed",
    "bottleman",
    "bouche",
    "boudreau",
    "bouncy",
    "bounder",
    "bourcier",
    "bourgeois",
    "bourguig",
    "bourlet",
    "bouse",
    "boutade",
    "bouto",
    "bouzouki",
    "bovid",
    "bowers",
    "bowwow",
    "bowyer",
    "boyar",
    "boyscout",
    "brach",
    "brachial",
    "brack",
    "brackets",
    "braddock",
    "bradley",
    "braggers",
    "brahms",
    "brainwasher",
    "brainwork",
    "bramble",
    "brambles",
    "branch",
    "brancher",
    "brandice",
    "brandie",
    "brann",
    "brassard",
    "braunschweiger",
    "breaded",
    "breaker",
    "breakless",
    "breakthrough",
    "breakup",
    "breakwater",
    "breaster",
    "breaststroker",
    "breathless",
    "breeds",
    "breena",
    "breezing",
    "brekel",
    "brennand",
    "breton",
    "brevier",
    "briar",
    "bribe",
    "bridal",
    "bridgeman",
    "bridges",
    "briel",
    "brierley",
    "brigandine",
    "brightly",
    "brimborion",
    "brimful",
    "brims",
    "briolette",
    "british",
    "britt",
    "broadway",
    "brocade",
    "broccoli",
    "broch",
    "brockie",
    "brockman",
    "broddy",
    "brody",
    "brogan",
    "brogues",
    "broking",
    "bromes",
    "bronco",
    "bronk",
    "bronny",
    "bronze",
    "broody",
    "brooklyn",
    "broom",
    "broomer",
    "brose",
    "brownlee",
    "browsers",
    "bruise",
    "brushwood",
    "brushy",
    "brusque",
    "brutter",
    "brydges",
    "brynn",
    "bryophyte",
    "bubak",
    "bubales",
    "buccan",
    "buccina",
    "bucketman",
    "buckeye",
    "buckhorn",
    "buckling",
    "buckman",
    "buddie",
    "budding",
    "buder",
    "bufonidae",
    "bugbear",
    "buggyman",
    "bugler",
    "built",
    "bulanda",
    "bulbous",
    "bulgaria",
    "bulimia",
    "bullen",
    "bullhorn",
    "bullocks",
    "bumfs",
    "bumpier",
    "bumpy",
    "bunchberry",
    "bunni",
    "burek",
    "burel",
    "burger",
    "burgesses",
    "burgle",
    "burial",
    "burian",
    "burin",
    "burker",
    "burkey",
    "burley",
    "burlie",
    "burps",
    "burrower",
    "burrus",
    "burry",
    "busby",
    "busch",
    "business",
    "businesswoman",
    "busybody",
    "butcheress",
    "butta",
    "buttered",
    "butterfish",
    "butting",
    "buxom",
    "buyable",
    "bygones",
    "byran",
    "byroad",
    "byron",
    "bystreet",
    "caatinga",
    "cabala",
    "cabalist",
    "caballer",
    "caballero",
    "caban",
    "cabasset",
    "cabinda",
    "cablet",
    "caboodle",
    "cabral",
    "cabrio",
    "cacara",
    "cachet",
    "cadaveric",
    "caddle",
    "cadmium",
    "caelum",
    "caffeina",
    "cagatay",
    "caggy",
    "cajaput",
    "caking",
    "calabrese",
    "calamine",
    "calamite",
    "calcar",
    "calcine",
    "calculous",
    "calcutta",
    "calefacient",
    "calinda",
    "calista",
    "callery",
    "calumny",
    "calvary",
    "camacho",
    "camber",
    "cambeva",
    "cambre",
    "cambrian",
    "cambridge",
    "camden",
    "camelus",
    "cameo",
    "camion",
    "campbell",
    "camper",
    "campo",
    "camused",
    "canadian",
    "canamo",
    "canberra",
    "candida",
    "cangle",
    "canille",
    "cannibal",
    "cannibals",
    "cannikin",
    "canopus",
    "canossa",
    "cantabile",
    "cantalas",
    "cantaloupes",
    "cantharis",
    "canto",
    "cantoral",
    "canuck",
    "canyon",
    "capabler",
    "capelet",
    "capelin",
    "capistrano",
    "capitalism",
    "capitation",
    "caplin",
    "capozzi",
    "caprice",
    "capricornus",
    "capriole",
    "capturer",
    "capuche",
    "carambole",
    "carapo",
    "carapus",
    "carayan",
    "carbonite",
    "carburant",
    "carcass",
    "cardamom",
    "cardia",
    "cardiff",
    "carding",
    "career",
    "caregiver",
    "carella",
    "carful",
    "caribou",
    "caridean",
    "carinal",
    "carine",
    "carita",
    "carlean",
    "carlene",
    "carless",
    "carlet",
    "carlett",
    "carlock",
    "carlyle",
    "carmelite",
    "carmencita",
    "carmina",
    "carnations",
    "carnell",
    "carnivore",
    "caroli",
    "carolus",
    "caron",
    "carotid",
    "carpetbag",
    "carpool",
    "carrefour",
    "carrizo",
    "carted",
    "cartesian",
    "carving",
    "casas",
    "cascarilla",
    "casco",
    "caseloads",
    "casey",
    "cashana",
    "cashbox",
    "caslon",
    "cassady",
    "cassandra",
    "cassat",
    "cassolette",
    "cassondra",
    "castaways",
    "castell",
    "castilian",
    "castiron",
    "castles",
    "castling",
    "casualist",
    "catacombs",
    "catamenia",
    "catbird",
    "catchpole",
    "caterina",
    "caterpillars",
    "catfight",
    "catgut",
    "cathedrals",
    "catherina",
    "cathole",
    "catie",
    "cation",
    "cauchy",
    "cauliflower",
    "causes",
    "cautions",
    "cautivo",
    "cavalero",
    "cavatina",
    "cavish",
    "caxton",
    "cayapa",
    "cayuga",
    "ceara",
    "cebil",
    "cebus",
    "cecon",
    "cedric",
    "ceibo",
    "celadon",
    "celarent",
    "celature",
    "celebrations",
    "celebrity",
    "celesta",
    "cellmate",
    "celloid",
    "cellule",
    "celsius",
    "celtish",
    "cembalo",
    "cementation",
    "cenozoic",
    "cense",
    "censored",
    "centenary",
    "centered",
    "centrefolds",
    "centric",
    "centrist",
    "centuple",
    "cephalotripsy",
    "ceramics",
    "ceratium",
    "cercocebus",
    "ceremonies",
    "ceriel",
    "cerite",
    "cerulean",
    "ceruleum",
    "ceruse",
    "cessna",
    "chafe",
    "chaja",
    "chaka",
    "chakras",
    "chalon",
    "chalot",
    "chalton",
    "chalumeau",
    "chamade",
    "chameleons",
    "champ",
    "champing",
    "chancery",
    "chances",
    "chandan",
    "chander",
    "chandra",
    "chanson",
    "chanteuse",
    "chaotic",
    "chape",
    "chaperno",
    "chaperone",
    "chapfallen",
    "chaplin",
    "chappy",
    "charging",
    "charil",
    "charlene",
    "charline",
    "charlotte",
    "charmian",
    "charmless",
    "charta",
    "chasing",
    "chatelaine",
    "chaudhry",
    "chaudry",
    "chauffeur",
    "chaute",
    "chavender",
    "chaya",
    "cheapest",
    "chechen",
    "cheeking",
    "cheering",
    "cheetham",
    "chegre",
    "chela",
    "chelone",
    "chemistry",
    "chemoreceptor",
    "chenault",
    "cheng",
    "chengwei",
    "cheques",
    "cherimoya",
    "cherish",
    "cherished",
    "cherly",
    "cherlyn",
    "cherokee",
    "cherries",
    "cheryl",
    "chetan",
    "chewer",
    "chiasma",
    "chichi",
    "chicomecoatl",
    "chiefs",
    "chieftain",
    "chien",
    "chigger",
    "chignon",
    "childe",
    "childhood",
    "chilicote",
    "chimalapa",
    "chimbley",
    "chinamen",
    "chine",
    "chinho",
    "chinotti",
    "chipchap",
    "chipped",
    "chiquito",
    "chiropractor",
    "chiseler",
    "chisolm",
    "chital",
    "chitter",
    "chivalric",
    "chivalrous",
    "chocolatey",
    "choctaw",
    "choices",
    "choler",
    "choluteca",
    "choosey",
    "chops",
    "chorai",
    "chordate",
    "chores",
    "chotts",
    "chouser",
    "chrisman",
    "chrismary",
    "christea",
    "christel",
    "christia",
    "christianson",
    "christl",
    "christoffel",
    "chromo",
    "chrysant",
    "chrysanthemum",
    "chrysops",
    "chrystal",
    "chukchi",
    "chump",
    "chumps",
    "chunkiest",
    "chunmei",
    "churchill",
    "churinga",
    "churn",
    "cicatrix",
    "cigua",
    "cilice",
    "cimex",
    "cinderel",
    "cineraria",
    "cingulate",
    "cinque",
    "cinter",
    "circadian",
    "circle",
    "circulate",
    "ciscos",
    "cithara",
    "citral",
    "citrons",
    "civies",
    "civility",
    "claimed",
    "claimer",
    "clairvoyants",
    "clamber",
    "clamorous",
    "clandestine",
    "clarance",
    "clarified",
    "clarin",
    "clarita",
    "clason",
    "classical",
    "clatty",
    "claudius",
    "claustrum",
    "claval",
    "claws",
    "cleancut",
    "cleanup",
    "clemens",
    "clepsydra",
    "clerks",
    "clethra",
    "clevie",
    "clients",
    "cliffs",
    "clifton",
    "climbed",
    "climber",
    "clinic",
    "clink",
    "clipper",
    "clitia",
    "clockhouse",
    "clocks",
    "clockwise",
    "cloherty",
    "clone",
    "clonus",
    "close",
    "clote",
    "cloudage",
    "clubs",
    "coagulate",
    "coaly",
    "coarse",
    "coast",
    "coater",
    "coatimundi",
    "cobble",
    "cobblers",
    "cobbra",
    "cobol",
    "cobwebs",
    "cocain",
    "cocaine",
    "cochin",
    "cocker",
    "cockeyed",
    "cocking",
    "cocoa",
    "cocooned",
    "codebreaker",
    "codfish",
    "codicil",
    "coerced",
    "coeval",
    "coffeehouse",
    "coffin",
    "coffinmaker",
    "coghlan",
    "cogitate",
    "cohabitants",
    "cohesion",
    "coils",
    "cointer",
    "colder",
    "coldish",
    "colene",
    "coleridge",
    "coleslaw",
    "colias",
    "colicky",
    "collard",
    "colleague",
    "collegium",
    "collisions",
    "colloid",
    "cologs",
    "colon",
    "colorant",
    "colorate",
    "colorize",
    "colourblind",
    "colston",
    "colts",
    "columbic",
    "comart",
    "comate",
    "combaz",
    "combinator",
    "combo",
    "comfit",
    "comfort",
    "comfortable",
    "comix",
    "comma",
    "commanded",
    "commander",
    "commanding",
    "compatriot",
    "complement",
    "completer",
    "completing",
    "complex",
    "compliant",
    "complier",
    "compose",
    "compress",
    "compulsed",
    "computer",
    "computerise",
    "concentrate",
    "concentric",
    "concrete",
    "concussion",
    "condom",
    "conestoga",
    "confab",
    "confection",
    "confederacy",
    "confers",
    "confessions",
    "conform",
    "confucius",
    "confused",
    "conger",
    "congregate",
    "conjure",
    "connelly",
    "connexions",
    "conniption",
    "conquered",
    "conscience",
    "consciences",
    "consciousness",
    "conservative",
    "consolable",
    "console",
    "consolidated",
    "consonance",
    "constanta",
    "constitutions",
    "constructor",
    "consultant",
    "contemplative",
    "contemptible",
    "contemptuous",
    "content",
    "continuum",
    "contradiction",
    "contrary",
    "contribution",
    "controllers",
    "convair",
    "convected",
    "converse",
    "converses",
    "convict",
    "convolvulus",
    "coogan",
    "cookeys",
    "coolest",
    "cooling",
    "cooper",
    "copious",
    "copperfield",
    "copyist",
    "coquelicot",
    "coquet",
    "coquitlam",
    "corabelle",
    "coranto",
    "corban",
    "corbett",
    "cordilleran",
    "cordy",
    "coregonus",
    "corena",
    "corie",
    "corina",
    "corker",
    "corkey",
    "cormac",
    "cornea",
    "corners",
    "cornflake",
    "cornice",
    "cornishman",
    "cornual",
    "coronet",
    "corporates",
    "corporeal",
    "correal",
    "correct",
    "correy",
    "corrigan",
    "corruption",
    "corsair",
    "cortina",
    "corvettes",
    "corvo",
    "corydon",
    "cosimo",
    "cosine",
    "cosmetic",
    "cosmorama",
    "costard",
    "costarred",
    "costed",
    "costello",
    "costs",
    "costumes",
    "costuming",
    "cotte",
    "cottonmouth",
    "cottontail",
    "cotys",
    "couchy",
    "couma",
    "coumara",
    "countersink",
    "countess",
    "countor",
    "counts",
    "couplet",
    "cousins",
    "coved",
    "coward",
    "cowhand",
    "cowkeeper",
    "coxswain",
    "coyan",
    "coyish",
    "coyle",
    "cozen",
    "crabber",
    "crabby",
    "cradler",
    "craft",
    "cramped",
    "crane",
    "craney",
    "cranic",
    "crankless",
    "cranky",
    "crassula",
    "crataegus",
    "craving",
    "crawthumper",
    "craze",
    "creak",
    "creaks",
    "creatine",
    "creating",
    "creative",
    "creche",
    "credendum",
    "credenza",
    "creecy",
    "creen",
    "creeps",
    "cremone",
    "crepeau",
    "crescentia",
    "cricker",
    "crile",
    "criminologist",
    "crimp",
    "crinoid",
    "crinoline",
    "crista",
    "cristie",
    "cristina",
    "criterium",
    "criticaster",
    "criticism",
    "critter",
    "croci",
    "crocky",
    "cronet",
    "crony",
    "croppy",
    "croquette",
    "crosse",
    "crossways",
    "croteau",
    "crowds",
    "crozer",
    "crozier",
    "cruce",
    "cruche",
    "crucify",
    "crude",
    "cruise",
    "cruising",
    "crumble",
    "crumblier",
    "crumby",
    "crumpler",
    "crustacean",
    "crusty",
    "cruth",
    "cryptic",
    "cuarenta",
    "cuculla",
    "cudden",
    "cuerda",
    "cuirassier",
    "cuisse",
    "culebra",
    "culet",
    "culinary",
    "culled",
    "cumulate",
    "cunard",
    "cupholder",
    "cupper",
    "curate",
    "curcuma",
    "curcumas",
    "curley",
    "curupira",
    "curver",
    "curvilinear",
    "cushing",
    "cusick",
    "customer",
    "customize",
    "cusumano",
    "cutesy",
    "cutlass",
    "cutlet",
    "cutline",
    "cutthroat",
    "cyanic",
    "cybernetic",
    "cyborgs",
    "cycles",
    "cyclic",
    "cycloid",
    "cynthy",
    "cypher",
    "cypress",
    "cyprus",
    "cyrenaic",
    "cyros",
    "cytherea",
    "d-day",
    "daaboul",
    "dabber",
    "dacca",
    "dacian",
    "dadaism",
    "dahoon",
    "daimen",
    "dairylea",
    "dairyman",
    "daisi",
    "dalcassian",
    "dalmatian",
    "damaged",
    "damaris",
    "damil",
    "damnation",
    "damocles",
    "damping",
    "danceable",
    "dangersome",
    "dangle",
    "danie",
    "danielak",
    "danize",
    "danny",
    "daphna",
    "dapped",
    "darkener",
    "darkens",
    "darker",
    "darkies",
    "darkness",
    "darnel",
    "darrack",
    "darrel",
    "darrin",
    "darshi",
    "dartboard",
    "dartman",
    "dashed",
    "dastardly",
    "databank",
    "dates",
    "datos",
    "datura",
    "daube",
    "dauphine",
    "davey",
    "davidh",
    "davidian",
    "dawne",
    "dawns",
    "daylights",
    "dayman",
    "daymare",
    "dayspring",
    "dayton",
    "dazing",
    "deadlocked",
    "deanery",
    "dearaujo",
    "deathy",
    "deaved",
    "debased",
    "debater",
    "debilitate",
    "debride",
    "debus",
    "decaf",
    "decafid",
    "decagon",
    "decatur",
    "decayed",
    "deceive",
    "decembrist",
    "decency",
    "decil",
    "decimate",
    "decision",
    "deckchair",
    "deckers",
    "deckie",
    "decking",
    "decolor",
    "decoupage",
    "decries",
    "decubitus",
    "dedicates",
    "deeann",
    "deena",
    "deere",
    "deermeat",
    "defeatism",
    "defect",
    "defensor",
    "defiant",
    "defined",
    "deflatable",
    "defloration",
    "deforestation",
    "deformative",
    "deformity",
    "defrance",
    "degenerative",
    "dehaan",
    "dehydration",
    "deidre",
    "deifier",
    "deipnophobia",
    "dejan",
    "dejected",
    "dekko",
    "dekle",
    "delage",
    "delaminate",
    "delano",
    "delat",
    "delawn",
    "delayer",
    "delight",
    "delila",
    "deliquescent",
    "delirious",
    "deliver",
    "delly",
    "delmor",
    "deloris",
    "delphinite",
    "delroy",
    "deluxe",
    "demarch",
    "demeo",
    "demijohn",
    "demimark",
    "demir",
    "demised",
    "demiss",
    "demons",
    "demur",
    "dendral",
    "denes",
    "denice",
    "dennie",
    "denny",
    "denominate",
    "dentin",
    "dentine",
    "depark",
    "depict",
    "depicting",
    "deposit",
    "depraved",
    "deprive",
    "deranger",
    "derates",
    "derelict",
    "dermis",
    "dervish",
    "descended",
    "desdemon",
    "desecration",
    "deserving",
    "designers",
    "desirae",
    "desires",
    "desiri",
    "desistance",
    "desolates",
    "despoiler",
    "despot",
    "dessa",
    "dessain",
    "destructive",
    "detailer",
    "determinant",
    "detin",
    "detlev",
    "detonations",
    "detour",
    "detours",
    "detur",
    "deuce",
    "deuteron",
    "deutzia",
    "devaluation",
    "developed",
    "devera",
    "devest",
    "deviated",
    "devious",
    "devoir",
    "devotes",
    "devow",
    "dewberry",
    "dewdrop",
    "dewit",
    "dewitte",
    "dextro",
    "dhangar",
    "dhava",
    "dhoul",
    "diablo",
    "diagnose",
    "diaphane",
    "diccon",
    "dichter",
    "dickerson",
    "dickford",
    "didactic",
    "didani",
    "didie",
    "didine",
    "didus",
    "dielike",
    "diena",
    "diene",
    "dieri",
    "diets",
    "differences",
    "differentiator",
    "diffraction",
    "digestives",
    "dight",
    "dijian",
    "dikkop",
    "diktat",
    "dilated",
    "dilbert",
    "dimension",
    "dimitra",
    "dined",
    "dingar",
    "dingbats",
    "dingley",
    "dinkey",
    "dinner",
    "dinners",
    "dinny",
    "dinosaur",
    "dipak",
    "dipietro",
    "diplopod",
    "diptera",
    "direction",
    "directory",
    "dirigent",
    "dirtbird",
    "disadventure",
    "disappear",
    "disappearance",
    "disazo",
    "disbelief",
    "discolor",
    "disconsolate",
    "discordia",
    "discording",
    "discount",
    "discoverer",
    "discovert",
    "discriminant",
    "disembodied",
    "disgrace",
    "disheart",
    "dislocator",
    "dismember",
    "disorderer",
    "disorient",
    "dispassioned",
    "dispatched",
    "dispirited",
    "display",
    "displayer",
    "dissave",
    "distad",
    "distances",
    "distinctive",
    "disturber",
    "ditch",
    "ditone",
    "ditto",
    "divergence",
    "dives",
    "divest",
    "divide",
    "divinely",
    "diviner",
    "diving",
    "divots",
    "divus",
    "diwata",
    "djins",
    "docherty",
    "docken",
    "documentary",
    "dodie",
    "dogged",
    "dogie",
    "doglike",
    "dogtag",
    "doily",
    "doina",
    "dollop",
    "dolman",
    "dolmen",
    "dolomites",
    "dolphin",
    "domination",
    "dominiqu",
    "dominium",
    "donal",
    "donga",
    "donica",
    "donned",
    "donnie",
    "donning",
    "donohue",
    "doodad",
    "doodlebug",
    "doolittle",
    "doree",
    "dorelia",
    "dories",
    "dorsal",
    "dorser",
    "dorter",
    "dosanjh",
    "dosers",
    "dosser",
    "dotan",
    "dotes",
    "dotson",
    "dotted",
    "double",
    "doubledecker",
    "doubts",
    "dougall",
    "douping",
    "doused",
    "dovelet",
    "dower",
    "downbeat",
    "downcome",
    "downless",
    "downs",
    "downtown",
    "downturn",
    "dowse",
    "draconis",
    "dragnet",
    "dragoman",
    "drainage",
    "drainer",
    "drainpipe",
    "drakes",
    "drane",
    "draughtsman",
    "dreader",
    "dreads",
    "dreaminess",
    "dreamworld",
    "dredges",
    "dreep",
    "drench",
    "drenched",
    "drescher",
    "dresel",
    "dress",
    "dresser",
    "dreyfus",
    "dreyfuss",
    "dribbling",
    "drier",
    "drink",
    "drive",
    "drizzly",
    "dropped",
    "dropping",
    "dropsy",
    "droud",
    "drover",
    "drseuss",
    "drugless",
    "drumlin",
    "drumline",
    "drunks",
    "dryas",
    "dubose",
    "duckers",
    "duckies",
    "ducking",
    "duckwalk",
    "duello",
    "duenas",
    "dulcea",
    "dullhead",
    "dulosis",
    "dumas",
    "dumbs",
    "dumper",
    "dunder",
    "dungs",
    "dunsmore",
    "dupable",
    "dupes",
    "dupree",
    "durango",
    "duress",
    "durmast",
    "durovic",
    "durrell",
    "dusky",
    "dustbox",
    "dustman",
    "dustuck",
    "dusun",
    "dutiful",
    "dwale",
    "dweller",
    "dynamism",
    "dynamistic",
    "dyslogistic",
    "dyspepsia",
    "dziemian",
    "eamonn",
    "eardrum",
    "earles",
    "earlier",
    "earlock",
    "earmark",
    "earring",
    "earrings",
    "earth",
    "earthenware",
    "eastland",
    "ebenezer",
    "eccentrics",
    "echard",
    "echinus",
    "eckler",
    "eclair",
    "econometrics",
    "ecstasy",
    "ecthyma",
    "edenize",
    "edlin",
    "edsel",
    "eduard",
    "educable",
    "edwin",
    "eeeeee",
    "efficacious",
    "efficience",
    "efficient",
    "egality",
    "eggfish",
    "egipto",
    "egret",
    "eidetic",
    "eikon",
    "eladio",
    "elanor",
    "elasmosaur",
    "elastica",
    "elberta",
    "elbow",
    "elding",
    "eldreth",
    "eleanore",
    "eleazar",
    "electrified",
    "electrocute",
    "electrodes",
    "electrolysis",
    "electrophone",
    "electrostatic",
    "electrotechnics",
    "elegances",
    "elephas",
    "elevator",
    "elevatory",
    "elfreda",
    "elgar",
    "elianora",
    "elided",
    "eliminable",
    "eliminator",
    "elinor",
    "elizabethan",
    "elkins",
    "eller",
    "ellie",
    "elsey",
    "elusions",
    "elusory",
    "elute",
    "elvera",
    "elvira",
    "elysia",
    "emacs",
    "emball",
    "embark",
    "embay",
    "embittered",
    "emblem",
    "embrica",
    "embryon",
    "emera",
    "emilee",
    "emilios",
    "eminence",
    "emirate",
    "emissary",
    "emmaline",
    "emmett",
    "emmey",
    "emogene",
    "emollient",
    "emond",
    "emory",
    "empathetic",
    "empathic",
    "emperors",
    "empress",
    "enamels",
    "enchanter",
    "encloud",
    "endang",
    "endeavours",
    "enderle",
    "endosteum",
    "endrys",
    "enforces",
    "engineman",
    "engle",
    "englishmen",
    "engrave",
    "engravers",
    "enhancer",
    "eniac",
    "enkidu",
    "ennead",
    "enrapt",
    "enrica",
    "enriquet",
    "enshroud",
    "ensign",
    "entach",
    "entails",
    "ental",
    "enteron",
    "entomb",
    "envelop",
    "enviable",
    "ephemeron",
    "ephialtes",
    "ephyra",
    "epical",
    "epicenter",
    "epicentral",
    "episcopalian",
    "epistropheus",
    "epomophorus",
    "equalling",
    "equilibrio",
    "equivocal",
    "erasement",
    "erastus",
    "erbach",
    "erection",
    "erena",
    "ergal",
    "ericsson",
    "eriksson",
    "erithacus",
    "erlenmeyer",
    "ermey",
    "erminia",
    "ernestine",
    "erode",
    "erroll",
    "erwin",
    "erykah",
    "erythrocyte",
    "escalus",
    "esclavage",
    "escobilla",
    "eskar",
    "eskimo",
    "esmond",
    "esparto",
    "esposito",
    "essed",
    "esson",
    "estelle",
    "ester",
    "estes",
    "estevam",
    "estherian",
    "estrange",
    "esurience",
    "etalon",
    "etheline",
    "ethelyn",
    "ether",
    "etherize",
    "ethically",
    "eulalia",
    "eulalie",
    "eunice",
    "euonymus",
    "euphonism",
    "euphori",
    "eurasian",
    "evangelian",
    "evaporative",
    "evened",
    "evens",
    "evensong",
    "everyman",
    "everyone",
    "evoked",
    "evokes",
    "evzone",
    "ewing",
    "exacta",
    "exaggerating",
    "excalibur",
    "excavating",
    "except",
    "exciton",
    "excitor",
    "exclude",
    "excoriation",
    "executant",
    "executed",
    "exegesis",
    "exegete",
    "exert",
    "exfoliate",
    "exhibits",
    "existential",
    "exophoria",
    "exorcise",
    "exordia",
    "exotic",
    "expatriate",
    "expiry",
    "explains",
    "exploiters",
    "explorations",
    "explorers",
    "explosion",
    "expone",
    "exponent",
    "exposition",
    "expresso",
    "extemporaneity",
    "externity",
    "extraneous",
    "extravert",
    "extremal",
    "eyeball",
    "eyeballs",
    "eyebrow",
    "eyeglasses",
    "eyelashes",
    "eyeline",
    "eyeshot",
    "eyewink",
    "ezekiel",
    "fabio",
    "fabler",
    "fabrizio",
    "faceman",
    "facer",
    "factice",
    "facto",
    "fagan",
    "fagottino",
    "fahey",
    "failsafe",
    "fairish",
    "fairless",
    "fairy",
    "falchion",
    "faliscan",
    "fallacy",
    "faller",
    "fanes",
    "fanner",
    "fantasize",
    "fatback",
    "fatidical",
    "feels",
    "ferrous",
    "ferule",
    "fewer",
    "fiction",
    "fiducial",
    "filmer",
    "findings",
    "firearms",
    "flabs",
    "flamer",
    "flamingo",
    "flamingos",
    "flatland",
    "flattop",
    "fletcher",
    "flightless",
    "flits",
    "fluffs",
    "flycatcher",
    "fogger",
    "foothill",
    "forehead",
    "forger",
    "forgotten",
    "fortepiano",
    "fortes",
    "fortuneteller",
    "forzando",
    "frazzled",
    "freemen",
    "friseur",
    "fugio",
    "funkia",
    "gaffe",
    "gallican",
    "gargantuan",
    "gasped",
    "gecko",
    "geology",
    "geometry",
    "gibed",
    "giggled",
    "gingiva",
    "girlies",
    "gizzard",
    "glasses",
    "glioma",
    "globefish",
    "gloms",
    "glucose",
    "glycerol",
    "going",
    "goodwill",
    "gooses",
    "gouger",
    "graciousness",
    "granulator",
    "grasper",
    "graybeard",
    "greener",
    "greengage",
    "gregariousness",
    "greylag",
    "grievance",
    "griffin",
    "gript",
    "groggy",
    "guars",
    "guernsey",
    "gulley",
    "gummers",
    "gunslinger",
    "habanera",
    "hadron",
    "hadrons",
    "halbert",
    "haler",
    "handbasket",
    "hangfire",
    "hardline",
    "harmonious",
    "harpy",
    "harrier",
    "hautbois",
    "havens",
    "havocs",
    "headwaiter",
    "heartbreak",
    "heeler",
    "hellion",
    "hematite",
    "hemoglobin",
    "henna",
    "herbivore",
    "heterosexual",
    "highbrows",
    "hillock",
    "hilum",
    "hocus",
    "hoers",
    "hoising",
    "hollyhocks",
    "holts",
    "hometown",
    "hoops",
    "hopeful",
    "hubcaps",
    "hubristic",
    "huddled",
    "humiliation",
    "hydroponics",
    "idiots",
    "idolatrous",
    "illegal",
    "imagining",
    "imbued",
    "immerging",
    "immolator",
    "immune",
    "impious",
    "improve",
    "impulsively",
    "inbounds",
    "incarnate",
    "incognitos",
    "indecency",
    "index",
    "indexical",
    "indris",
    "indulger",
    "industry",
    "inequality",
    "inexplicit",
    "infest",
    "ingots",
    "inspector",
    "inspired",
    "installation",
    "insular",
    "integrate",
    "intercom",
    "intercoms",
    "interflow",
    "internal",
    "interregnum",
    "invigoration",
    "inviscid",
    "invite",
    "irater",
    "iridescent",
    "ironwork",
    "irradiation",
    "irregular",
    "irrelevantly",
    "jacanas",
    "jalop",
    "janitors",
    "jazzing",
    "jellyfish",
    "jenny",
    "jeopardise",
    "jerries",
    "jocund",
    "jollier",
    "jostler",
    "jujube",
    "juking",
    "kabiki",
    "kerning",
    "keyless",
    "kibbutz",
    "killing",
    "kilohertz",
    "kilts",
    "kindle",
    "kingdoms",
    "kinkajou",
    "kiwifruit",
    "kolhoz",
    "kooks",
    "kwashiorkor",
    "labyrinth",
    "laceration",
    "lacrosse",
    "lambs",
    "lamebrain",
    "landlords",
    "landmark",
    "latria",
    "launce",
    "laureate",
    "lawless",
    "lazying",
    "leafy",
    "lease",
    "lectures",
    "lefty",
    "lemon",
    "lesbians",
    "levering",
    "levied",
    "libra",
    "lieve",
    "lifer",
    "lingering",
    "lining",
    "lipomas",
    "litotes",
    "lobations",
    "locoed",
    "lollygag",
    "loment",
    "longhair",
    "longlines",
    "longshoremen",
    "loudness",
    "louie",
    "loupes",
    "lovebirds",
    "lovers",
    "lucidity",
    "ludicrous",
    "lumen",
    "lymph",
    "macrophage",
    "madonna",
    "madras",
    "magenta",
    "magistral",
    "maidenhead",
    "maile",
    "maimed",
    "malices",
    "malpractice",
    "malvasia",
    "manche",
    "manning",
    "mariachi",
    "marijuanas",
    "martello",
    "massa",
    "masseurs",
    "masterly",
    "masterstroke",
    "mastoid",
    "mavie",
    "meanwhile",
    "meatmen",
    "mechanic",
    "medical",
    "mediums",
    "megalomaniac",
    "merchants",
    "merrily",
    "metalsmith",
    "metatarsi",
    "meteorology",
    "microtome",
    "miffs",
    "migraine",
    "millage",
    "milled",
    "mindsets",
    "minima",
    "mirth",
    "mistermed",
    "mitten",
    "mixers",
    "mocha",
    "moggie",
    "mogul",
    "moguls",
    "moirai",
    "molar",
    "monastery",
    "mongeese",
    "monikers",
    "monkeyshines",
    "monomania",
    "moors",
    "moped",
    "moralistic",
    "moratoria",
    "mosks",
    "mosso",
    "mothball",
    "motility",
    "motivator",
    "motorman",
    "mousse",
    "mullite",
    "multijet",
    "musician",
    "mutine",
    "myalgic",
    "named",
    "namelessly",
    "nappy",
    "naturalistic",
    "naturalists",
    "navies",
    "nepenthe",
    "netlike",
    "neuter",
    "newborns",
    "nibble",
    "ninjas",
    "nobodies",
    "nodder",
    "nonbelief",
    "normalcy",
    "normality",
    "norms",
    "novelette",
    "nympholept",
    "obscenity",
    "obsoletes",
    "oddballs",
    "offside",
    "offsprings",
    "omnivora",
    "oncogene",
    "onions",
    "onomatopoeia",
    "oppose",
    "orache",
    "organismal",
    "orthodox",
    "ostraca",
    "outlandish",
    "outlet",
    "outside",
    "overdogs",
    "overshadow",
    "overview",
    "oxeyes",
    "oxides",
    "ozonic",
    "pacas",
    "padauk",
    "pager",
    "painful",
    "palindrome",
    "palmar",
    "palmetto",
    "pancreatitis",
    "panicked",
    "panthers",
    "pantomime",
    "papain",
    "papal",
    "paperweight",
    "papillon",
    "paradise",
    "parlay",
    "parsed",
    "parser",
    "parterres",
    "paseos",
    "passe",
    "passwords",
    "pastorale",
    "patness",
    "patter",
    "pauses",
    "peacemaker",
    "pecky",
    "pediatric",
    "pedicab",
    "pedlers",
    "pedophile",
    "peery",
    "penning",
    "pennyroyal",
    "pensione",
    "penstemon",
    "peponidas",
    "perfecto",
    "peroxid",
    "perturbation",
    "petal",
    "pewit",
    "phosphate",
    "photon",
    "picoline",
    "pinnacle",
    "pinole",
    "pinwheel",
    "pioneering",
    "piperazine",
    "pipsqueak",
    "piques",
    "pirogue",
    "pisiform",
    "placebos",
    "plasmoid",
    "playland",
    "please",
    "pleonastic",
    "plimsol",
    "plotter",
    "poesy",
    "poetess",
    "politics",
    "polled",
    "pommies",
    "pondering",
    "pooled",
    "porkpie",
    "positive",
    "postcard",
    "postfire",
    "poundage",
    "pourer",
    "poverty",
    "preatomic",
    "precognition",
    "preview",
    "primeval",
    "princesse",
    "prisons",
    "processor",
    "proclaimed",
    "progression",
    "prophecy",
    "prosecutor",
    "prostitute",
    "protesting",
    "protomartyr",
    "prune",
    "pseudonym",
    "puffs",
    "pullover",
    "pulvinus",
    "purdah",
    "pyrogenicity",
    "pyrrhic",
    "qualm",
    "quarts",
    "quebrachos",
    "quietens",
    "rabboni",
    "racker",
    "racon",
    "raincoat",
    "raphia",
    "ratiocinator",
    "razers",
    "realest",
    "reaper",
    "reasonable",
    "receipt",
    "recipe",
    "redips",
    "redout",
    "reefer",
    "reflection",
    "refuted",
    "regularity",
    "reinvent",
    "relaxing",
    "relentless",
    "religion",
    "removed",
    "repair",
    "replenished",
    "represented",
    "requestor",
    "restrict",
    "retort",
    "retractor",
    "reversible",
    "revulsion",
    "rhizobium",
    "riced",
    "ridicule",
    "rille",
    "rolfers",
    "rooky",
    "roseola",
    "rotavirus",
    "rotted",
    "rousseau",
    "routeman",
    "rovers",
    "roving",
    "rubeola",
    "rumpus",
    "runaround",
    "russet",
    "ruthlessness",
    "rykes",
    "sabed",
    "saddlers",
    "salads",
    "sales",
    "saltire",
    "saltish",
    "samaras",
    "samba",
    "sanatorium",
    "sanded",
    "santalol",
    "sapid",
    "sapient",
    "sarges",
    "sassiest",
    "satsumas",
    "sawbucks",
    "sawer",
    "sawing",
    "saxes",
    "scabs",
    "scaled",
    "scallywags",
    "scalp",
    "scalper",
    "scampers",
    "scams",
    "scandic",
    "scarabaeus",
    "scarabs",
    "scarcity",
    "scarlet",
    "scarped",
    "scatt",
    "scattershot",
    "scheduler",
    "schematic",
    "scheme",
    "schipperke",
    "schizophrenic",
    "schlock",
    "schmelze",
    "schnaps",
    "schnecken",
    "schrik",
    "schuit",
    "schuss",
    "science",
    "sciences",
    "scimitar",
    "scintillator",
    "scions",
    "scirocco",
    "scissor",
    "sclera",
    "scoop",
    "scoot",
    "scope",
    "scorch",
    "scorching",
    "scorer",
    "scorner",
    "scouth",
    "scrag",
    "scraps",
    "scrimpy",
    "scriptures",
    "scrod",
    "scroll",
    "scrounge",
    "scruff",
    "sculpt",
    "scumbag",
    "scumble",
    "scurry",
    "scuzzy",
    "scythe",
    "scythed",
    "seaside",
    "seatbelts",
    "seated",
    "seaters",
    "seawan",
    "seaward",
    "seaweeds",
    "sebasic",
    "seconds",
    "secret",
    "secretive",
    "secretor",
    "section",
    "sectored",
    "sedated",
    "sedater",
    "sedatives",
    "sedimentary",
    "seduced",
    "seemly",
    "seersucker",
    "segment",
    "segued",
    "segueing",
    "seiner",
    "seining",
    "seise",
    "seiser",
    "seism",
    "selachian",
    "seldom",
    "sellouts",
    "selvas",
    "semblance",
    "senary",
    "senna",
    "sensorium",
    "separated",
    "septs",
    "serines",
    "setae",
    "sexualized",
    "shakable",
    "shaly",
    "shaman",
    "shamus",
    "shandies",
    "shaping",
    "shawm",
    "sheeting",
    "shelver",
    "sheriff",
    "shott",
    "shove",
    "sided",
    "sidetracked",
    "sighless",
    "signal",
    "signor",
    "silanes",
    "silhouette",
    "silicon",
    "silly",
    "simars",
    "simmer",
    "simoniac",
    "simoom",
    "simpleminded",
    "singly",
    "sirens",
    "sizzles",
    "skellum",
    "sketch",
    "skims",
    "skinkers",
    "slammer",
    "slaters",
    "sledged",
    "slinking",
    "smackers",
    "smarmy",
    "smokers",
    "smoldering",
    "snails",
    "snakebite",
    "snakey",
    "snarling",
    "snips",
    "snobby",
    "snorkeling",
    "snowdrop",
    "snowdrops",
    "soave",
    "socked",
    "sokols",
    "solarise",
    "soling",
    "solitude",
    "solver",
    "sophies",
    "soundless",
    "sourish",
    "spacial",
    "spancel",
    "spawners",
    "species",
    "spectators",
    "speculate",
    "speer",
    "spelter",
    "spent",
    "spice",
    "spieler",
    "spiritually",
    "splay",
    "splendent",
    "splendiferous",
    "sportsmen",
    "sprit",
    "squeal",
    "squirting",
    "stacker",
    "stallions",
    "stampeded",
    "staple",
    "starship",
    "starvation",
    "stele",
    "stelic",
    "stengah",
    "steric",
    "stirrup",
    "stoop",
    "stoper",
    "streamliner",
    "street",
    "strenuous",
    "stripling",
    "stubbles",
    "stupid",
    "stylar",
    "subatom",
    "subfusc",
    "subspace",
    "sucks",
    "suddens",
    "suedes",
    "suing",
    "sundering",
    "sunshiny",
    "supershow",
    "superstardom",
    "supervene",
    "surcease",
    "surrey",
    "suspected",
    "suzerain",
    "sweatband",
    "switchers",
    "syllogist",
    "sylvan",
    "syphered",
    "systemic",
    "tabloids",
    "tafia",
    "tailcoat",
    "takeout",
    "tapadera",
    "tarantas",
    "tariff",
    "tastier",
    "tasty",
    "taunting",
    "tawer",
    "taxidermist",
    "teenage",
    "telegrams",
    "telephoto",
    "tenant",
    "tenons",
    "terrorized",
    "tessera",
    "testimony",
    "tewing",
    "thanking",
    "theist",
    "thieving",
    "thorax",
    "thousand",
    "thrilling",
    "throne",
    "thrower",
    "thrush",
    "thunderstorm",
    "thyroxine",
    "tilak",
    "tilled",
    "tinfoil",
    "tingler",
    "tinkle",
    "tipple",
    "tobacconist",
    "toking",
    "tommed",
    "toner",
    "tonic",
    "tonights",
    "tonsured",
    "tootle",
    "topkick",
    "torot",
    "torpedos",
    "torpor",
    "touchable",
    "toughie",
    "tovarish",
    "toxicologist",
    "trachea",
    "tracing",
    "tracking",
    "tradition",
    "transact",
    "transform",
    "transistors",
    "tressed",
    "trigs",
    "trinity",
    "trioxide",
    "tritoma",
    "trucker",
    "trust",
    "tuner",
    "tupelo",
    "turmeric",
    "tutorial",
    "tzarina",
    "ultrasonic",
    "umiaq",
    "unaccountably",
    "unacted",
    "undercount",
    "underlain",
    "underling",
    "undermined",
    "unladylike",
    "unmercifully",
    "unrecorded",
    "unreliable",
    "unremarkable",
    "unshackled",
    "unspecific",
    "uppercut",
    "utilize",
    "utilizer",
    "validity",
    "valours",
    "vanished",
    "vaultier",
    "vegan",
    "veining",
    "vermillions",
    "vertebras",
    "vibrio",
    "viceless",
    "videotape",
    "viking",
    "vilification",
    "vineal",
    "vises",
    "vituperative",
    "vivers",
    "vocalist",
    "vodkas",
    "vomer",
    "voyageurs",
    "waddie",
    "wafer",
    "waggle",
    "waiting",
    "wakened",
    "walies",
    "wallet",
    "walrus",
    "wampum",
    "wangler",
    "wanton",
    "wared",
    "warship",
    "warthogs",
    "washy",
    "watercolors",
    "waylay",
    "weedlike",
    "westwards",
    "whammo",
    "whitier",
    "whoever",
    "wimple",
    "wingy",
    "winterberry",
    "wipers",
    "wivern",
    "woesome",
    "wonderful",
    "wonders",
    "woofs",
    "workbench",
    "worker",
    "wound",
    "woundless",
    "wrath",
    "wries",
    "yaird",
    "yealing",
    "yeanling",
    "yucca",
    "zaffre",
    "zapateo",
    "zappiest",
    "zelkova",
    "zillionaire",
    "zooks",
]

SOCIAL_NETWORKS = {
    'facebook': 'facebook.com/{}',
    'twitter': 'twitter.com/{}',
    'instagram': 'instagram.com/{}',
    'vk': 'vk.com/{}',
}



from typing import Optional

from mimesis.data import (CALLING_CODES, CONTINENT_CODES, COUNTRIES_ISO,
                          SHORTENED_ADDRESS_FMT)
from mimesis.enums import CountryCode
from mimesis.providers.base import BaseProvider
from mimesis.utils import pull


class Address(BaseProvider):
    """Class for generate fake address data."""

    def __init__(self, *args, **kwargs):
        """
        :param str locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self._data = pull('address.json', self.locale)

    def street_number(self, maximum: int = 1400) -> str:
        """Generate a random street number.

        :param int maximum: Maximum value.
        :return: Street number.

        :Example:
            134.
        """
        number = self.random.randint(1, int(maximum))
        return '{}'.format(number)

    def street_name(self) -> str:
        """Get a random street name.

        :return: Street name.

        :Example:
           Candlewood.
        """
        names = self._data['street'].get('name')
        return self.random.choice(names)

    def street_suffix(self) -> str:
        """Get a random street suffix.

        :return: Street suffix.

        :Example:
            Alley.
        """
        suffixes = self._data['street'].get('suffix')
        return self.random.choice(suffixes)

    def address(self) -> str:
        """Get a random full address (include Street number, suffix and name).

        :return: Full address.

        :Example:
            5 Central Sideline.
        """
        fmt = self._data['address_fmt']

        st_num = self.street_number()
        st_name = self.street_name()

        if self.locale in SHORTENED_ADDRESS_FMT:
            return fmt.format(
                st_num=st_num,
                st_name=st_name,
            )

        if self.locale == 'ja':
            cities = self._data['city']
            city = self.random.choice(cities)

            n, nn, nnn = self.random.randints(3, 1, 100)
            return fmt.format(city=city, n=n, nn=nn, nnn=nnn)

        return fmt.format(
            st_num=st_num,
            st_name=st_name,
            st_sfx=self.street_suffix(),

        )

    def state(self, abbr: bool = False) -> str:
        """Get a random administrative district of country.

        :param bool abbr: Return ISO 3166-2 code.
        :return: Administrative district.

        :Example:
            Alabama (for locale `en`).
        """
        key = 'abbr' if abbr else 'name'
        states = self._data['state'].get(key)
        return self.random.choice(states)

    def region(self, abbr: bool = False) -> str:
        """Get a random region.

        :param bool abbr: Return ISO 3166-2 code.
        :return: State.
        """
        return self.state(abbr)

    def province(self, abbr: bool = False) -> str:
        """Get a random province.

        :param bool abbr: Return ISO 3166-2 code.
        :return: Province.
        """
        return self.state(abbr)

    def federal_subject(self, abbr: bool = False) -> str:
        """Get a random region.

        :param bool abbr: Return ISO 3166-2 code.
        :return: Federal subject.
        """
        return self.state(abbr)

    def postal_code(self) -> str:
        """Generate a postal code for current locale.

        :return: Postal code.

        :Example:
            389213
        """

        mask = self._data['postal_code_fmt']
        return self.random.custom_code(mask=mask)

    def country_iso_code(self, fmt: Optional[CountryCode] = None) -> str:
        """Get a random ISO code of country.

        :param fmt: Enum object CountryCode.
        :return: ISO Code.
        :raises KeyError: if fmt is not supported.

        :Example:
            DE
        """
        key = self._validate_enum(fmt, CountryCode)
        codes = COUNTRIES_ISO[key]
        return self.random.choice(codes)

    def country(self) -> str:
        """Get a random country.

        :return: The Country.

        :Example:
            Russia.
        """
        countries = self._data['country'].get('name')
        return self.random.choice(countries)

    def city(self) -> str:
        """Get a random city for current locale.

        :return: City name.

        :Example:
            Saint Petersburg.
        """
        cities = self._data['city']
        return self.random.choice(cities)

    def latitude(self) -> float:
        """Generate a random value of latitude (-90 to +90).

        :return: Value of longitude.

        :Example:
            -66.4214188124611
        """
        return self.random.uniform(-90, 90)

    def longitude(self) -> float:
        """Generate a random value of longitude (-180 to +180).

        :return: Value of longitude.

        :Example:
            112.18440260511943
        """
        return self.random.uniform(-180, 180)

    def coordinates(self) -> dict:
        """Generate random geo coordinates.

        :return: Dict with coordinates.

        :Example:
            {'latitude': 8.003968712834975,
            'longitude': 36.02811153405548}
        """
        coord = {
            'longitude': self.longitude(),
            'latitude': self.latitude(),
        }
        return coord

    def continent(self, code: bool = False) -> str:
        """Get a random continent name or continent
        code (code in international format).

        :param bool code: Return code of continent.
        :return: Continent name.

        :Example:
            Africa (en)
        """
        if code:
            return self.random.choice(CONTINENT_CODES)

        continents = self._data['continent']
        return self.random.choice(continents)

    def calling_code(self) -> str:
        """Get a random calling code of random country.

        :return: Calling code.

        :Example:
            +7
        """
        return self.random.choice(CALLING_CODES)



from typing import Optional

from mimesis.builtins.base import BaseSpecProvider
from mimesis.enums import Gender
from mimesis.utils import pull


class RussiaSpecProvider(BaseSpecProvider):
    """Specific data for Russian language"""

    def __init__(self):
        super().__init__()
        self._data = pull('builtin.json', 'ru')

    class Meta:
        name = 'russia_provider'

    def generate_sentence(self) -> str:
        """Generate sentence from the parts.

        :return: Sentence.
        """
        sentences = self._data['sentence']
        sentence = [
            self.random.choice(sentences[k]) for k
            in ('head', 'p1', 'p2', 'tail')
        ]
        return '{0} {1} {2} {3}'.format(*sentence)

    def patronymic(self, gender: Optional[Gender] = None) -> str:
        """Generate random patronymic name.

        :param gender: Gender of person.
        :type gender: str or int
        :return: Patronymic name.

        :Example:
            Алексеевна.
        """
        gender = self._validate_enum(gender, Gender)
        patronymics = self._data['patronymic'][gender]
        return self.random.choice(patronymics)

    def passport_series(self, year: int = None) -> str:
        """Generate random series of passport.

        :param year: Year of manufacture.
        :type year: int or None
        :return: Series.

        :Example:
            02 15.
        """
        if not year:
            year = self.random.randint(10, 18)

        region = self.random.randint(1, 99)
        return '{:02d} {}'.format(region, year)

    def passport_number(self) -> int:
        """Generate random passport number.

        :return: Number.

        :Example:
            560430
        """
        return self.random.randint(
            100000, 999999)

    def series_and_number(self) -> str:
        """Generate a random passport number and series.

        :return: Series and number.

        :Example:
            57 16 805199.
        """

        return '{}{}'.format(
            self.passport_series(),
            self.passport_number(),
        )

    def snils(self) -> str:
        """Generate Individual insurance account number (SNILS).
        This method does not generate SNILS using algorithm and
        it's mean that SNILS generated by this method can be invalid.

        :return: SNILS.

        :Example:
            451-952-540-41.
        """
        mask = '###-###-###-##'
        return self.random.custom_code(mask=mask)



import re
import string
from typing import Optional

from mimesis.data import CREDIT_CARD_NETWORKS
from mimesis.enums import CardType, Gender
from mimesis.exceptions import NonEnumerableError
from mimesis.providers.base import BaseProvider
from mimesis.providers.personal import Personal
from mimesis.utils import luhn_checksum


class Payment(BaseProvider):
    """Data related to payments"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__personal = Personal('en', seed=self.seed)

    def cid(self) -> int:
        """Generate a random CID code.

        :return: CID code.

        :Example:
            7452
        """
        return self.random.randint(1000, 9999)

    def paypal(self) -> str:
        """Generate a random PayPal account.

        :return: Email of PapPal user.

        :Example:
            wolf235@gmail.com
        """
        return self.__personal.email()

    def bitcoin_address(self) -> str:
        """Generate a random bitcoin address.

        :return: Bitcoin address.

        :Example:
            3EktnHQD7RiAE6uzMj2ZifT9YgRrkSgzQX
        """
        type_ = self.random.choice(['1', '3'])
        letters = string.ascii_letters + string.digits
        address = [self.random.choice(letters) for _ in range(33)]
        return type_ + ''.join(address)

    def ethereum_address(self) -> str:
        """Get random dummy Ethereum address.

        .. Note: The address will look like Ethereum address,
        but keep in mind that it is not the valid address.
        It is just a SHA1 hash with prefixed 0x.

        :return: Ethereum address.

        :Example:
            0xe8ece9e6ff7dba52d4c07d37418036a89af9698d
        """
        bits = self.random.getrandbits(160)
        address = bits.to_bytes(20, byteorder='big')
        return '0x' + address.hex()

    def credit_card_network(self) -> str:
        """Get random credit card network

        :return: Credit card network

        :Example:
            MasterCard
        """
        return self.random.choice(CREDIT_CARD_NETWORKS)

    def credit_card_number(self, card_type: Optional[CardType] = None) -> str:
        """Generate a random credit card number.

        :param str card_type: Issuing Network. Default is Visa.
        :return: Credit card number.
        :raises NotImplementedError: if cart_type is not supported.

        :Example:
            4455 5299 1152 2450
        """
        length = 16
        regex = re.compile('(\d{4})(\d{4})(\d{4})(\d{4})')

        if card_type is None:
            card_type = CardType.get_random_item()

        if card_type == CardType.VISA:
            number = self.random.randint(4000, 4999)
        elif card_type == CardType.MASTER_CARD:
            number = self.random.choice([
                self.random.randint(2221, 2720),
                self.random.randint(5100, 5500),
            ])
        elif card_type == CardType.AMERICAN_EXPRESS:
            number = self.random.choice([34, 37])
            length = 15
            regex = re.compile('(\d{4})(\d{6})(\d{5})')
        else:
            raise NonEnumerableError(CardType)

        str_num = str(number)
        while len(str_num) < length - 1:
            str_num += self.random.choice(string.digits)

        groups = regex.search(str_num + luhn_checksum(str_num))
        card = ' '.join(groups.groups())
        return card

    def credit_card_expiration_date(self, minimum: int = 16,
                                    maximum: int = 25) -> str:
        """Generate a random expiration date for credit card.

        :param int minimum: Date of issue.
        :param int maximum: Maximum of expiration_date.
        :return: Expiration date of credit card.

        :Example:
            03/19.
        """
        month = self.random.randint(1, 12)
        year = self.random.randint(minimum, maximum)
        return '{0:02d}/{1}'.format(month, year)

    def cvv(self) -> int:
        """Generate a random card verification value (CVV).

        :return: CVV code.

        :Example:
            324
        """
        return self.random.randint(100, 999)

    def credit_card_owner(self, gender: Optional[Gender] = None) -> dict:
        owner = {
            'credit_card': self.credit_card_number(),
            'expiration_date': self.credit_card_expiration_date(),
            'owner': self.__personal.full_name(gender=gender).upper(),
        }
        return owner



"""This module is provide internal util functions for mimesis
"""

import collections
import functools
import json
import ssl
from os import path
from typing import Mapping, Optional, Union
from urllib import request

from mimesis import config
from mimesis.exceptions import UnsupportedLocale
from mimesis.typing import JSON

__all__ = ['download_image', 'locale_info', 'setup_locale']

PATH = path.abspath(path.join(path.dirname(__file__), 'data'))


def locale_info(locale: str) -> str:
    """Return name (in english) or local name of the locale

    :param str locale: Locale abbreviation.
    :return: Locale name.
    :raises UnsupportedLocale: if locale is not supported.
    """
    locale = locale.lower()
    supported = config.SUPPORTED_LOCALES

    if locale not in supported:
        raise UnsupportedLocale(locale)

    return supported[locale]['name']


def luhn_checksum(num: str) -> str:
    """Calculate a checksum for num using the Luhn algorithm.

    :param str num: The number to calculate a checksum for as a string.
    :return: Checksum for number.
    """
    check = 0
    for i, s in enumerate(reversed(num)):
        sx = int(s)
        sx = sx * 2 if i % 2 == 0 else sx
        sx = sx - 9 if sx > 9 else sx
        check += sx
    return str(check * 9 % 10)


def update_dict(initial: JSON, other: Mapping) -> JSON:
    """Recursively update a dictionary.

    :param initial: Dict to update.
    :type initial: dict or list
    :param other: Dict to update from.
    :type other: Mapping
    :return: Updated dict.
    :rtype: dict
    """
    for key, value in other.items():
        if isinstance(value, collections.Mapping):
            r = update_dict(initial.get(key, {}), value)
            initial[key] = r
        else:
            initial[key] = other[key]
    return initial


@functools.lru_cache(maxsize=None)
def pull(file: str, locale: str = 'en') -> JSON:
    """Open json file file and get content from file and memorize result using
     lru_cache.

    :param str file: The name of file.
    :param str locale: Locale.
    :return: The content of the file.
    :rtype: dict
    :raises UnsupportedLocale: if locale is not supported.

    :Example:

        >>> from mimesis.utils import pull
        >>> en = pull(file='datetime.json', locale='en')
        >>> isinstance(en, dict)
        True
        >>> en['day']['abbr'][0]
        'Mon.'
    """

    def get_data(locale_name: str) -> JSON:
        """Pull JSON data from file.

        :param locale_name: Name of locale to pull.
        :return: Dict of data from file
        """
        file_path = path.join(PATH + '/' + locale_name, file)
        # Needs explicit encoding for Windows
        with open(file_path, 'r', encoding='utf8') as f:
            return json.load(f)

    locale = locale.lower()

    if locale not in config.SUPPORTED_LOCALES:
        raise UnsupportedLocale(locale)

    master_locale = locale.split('-')[0]
    data = get_data(master_locale)

    # Handle sub-locales
    if '-' in locale:
        data = update_dict(data, get_data(locale))

    return data


def download_image(url: str = '', save_path: str = '',
                   unverified_ctx: bool = False) -> Union[None, str]:
    """Download image and save in current directory on local machine.

    :param str url: URL to image.
    :param str save_path: Saving path.
    :param bool unverified_ctx: Create unverified context.
    :return: Image name.
    :rtype: str or None
    """
    if unverified_ctx:
        ssl._create_default_https_context = ssl._create_unverified_context

    if url is not None:
        image_name = url.rsplit('/')[-1]
        request.urlretrieve(url, save_path + image_name)
        return image_name
    return None


def setup_locale(locale: Optional[str] = None) -> str:
    """Setup locale to BaseProvider.

    :param str locale: Locale
    :return: Locale in lowercase.
    :raises UnsupportedLocale: if locales is not supported.
    """
    if not locale:
        return config.DEFAULT_LOCALE

    locale = locale.lower()
    if locale not in config.SUPPORTED_LOCALES:
        raise UnsupportedLocale(locale)

    return locale



"""
Mimesis
-------

**Mimesis** is a fast and easy to use the library for Python programming
language, which helps generate mock data for a variety of purposes in a
variety of languages. This data can be particularly useful during software
development and testing. For example, it could be used to populate a
testing database for a web application with user information such as
email addresses, usernames, first names, last names, etc.

Documentation
-------------

Mimesis is very simple to use, and the below examples should help you
get started. Complete documentation for Mimesis is available on `Read
the Docs`_.

.. _Read the Docs: http://mimesis.readthedocs.io/

Installation
------------

To install mimesis, simply use pip (or `pipenv <http://pipenv.org/>`_):

.. code:: bash

    ➜  ~ pip install mimesis

Also, you can install it manually:

.. code:: bash

    (env) ➜ python3 setup.py install
    # or
    (env) ➜ make install

Getting started
---------------

As we said above, this library is really easy to use. A simple usage
example is given below:

.. code:: python

    >>> from mimesis import Personal
    >>> from mimesis.enums import Gender
    >>> person = Personal('en')

    >>> person.full_name(gender=Gender.FEMALE)
    'Antonetta Garrison'

    >>> person.occupation()
    'Backend Developer'

    >>> for template in ('U_d', 'U-d', 'l_d', 'l-d'):
    ...     person.username(template=template)

    'Adders_1893'
    'Abdel-1888'
    'constructor_1884'
    'chegre-2051'

Locales
-------

You can specify a locale when creating providers and they will return data that is appropriate for
the language or country associated with that locale. `Mimesis` currently includes support
for `33 different locales <http://mimesis.readthedocs.io/locales.html>`_.

Data Providers
--------------

List of supported data providers available `here <http://mimesis.readthedocs.io/providers.html>`_

"""

import json
import os
import re
import sys
from distutils.core import setup
from os.path import abspath, dirname, exists, getsize, join, relpath, splitext
from shutil import rmtree

from setuptools import Command
from setuptools.command.test import test as TestCommand

VERSION_MINOR_MAX = 10
VERSION_MICRO_MAX = 10

here = abspath(dirname(__file__))

try:
    with open('dev_requirements.txt') as f:
        tests_requirements = f.read().splitlines()
except FileNotFoundError:
    tests_requirements = []

about = {}
# Get meta-data from __version__.py
with open(join(here, 'mimesis', '__version__.py')) as f:
    exec(f.read(), about)


class BaseCommand(Command):
    description = ''
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        pass


class Upload(BaseCommand):
    """Support setup.py upload."""

    def run(self):
        try:
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))
        os.system('twine upload dist/*')
        sys.exit()


class PyTest(TestCommand):
    """Custom command for running test using setup.py test"""

    user_options = [('pytest-args=', 'a', 'Arguments to pass to py.test')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        exit(errno)


class Minimizer(BaseCommand):
    """Minify content of all json file for all locales.
    """

    def initialize_options(self):
        """Find all files of all locales.
        """

        self.paths = []
        self.data_path = '/mimesis/data'
        self.separators = (',', ':')
        self.data_dir = here + self.data_path
        self.before_total = 0
        self.after_total = 0

        for root, _, files in os.walk(self.data_dir):
            for file in sorted(files):
                if splitext(file)[1] == '.json':
                    self.paths.append(
                        join(
                            relpath(root, self.data_dir),
                            file,
                        )
                    )

    @staticmethod
    def size_of(num):
        for unit in ['B', 'KB', 'MB']:
            if abs(num) < 1024.0:
                return "%3.1f%s" % (num, unit)
            num = num / 1024.0
        return "%.1f" % num

    def minify(self, file_path):
        size_before = getsize(file_path)
        self.before_total += size_before
        size_before = self.size_of(size_before)

        with open(file_path, 'r', 1) as f:
            json_text = json.loads(f.read())
            minimized = json.dumps(
                json_text, separators=self.separators, ensure_ascii=False)

        if len(file_path) > 0:
            output_path = abspath(file_path)
            abs_path = abspath(dirname(output_path))

            if not exists(abs_path):
                os.makedirs(abs_path)

            with open(output_path, 'w+', 1) as f:
                f.write(minimized)

        size_after = getsize(file_path)
        self.after_total += size_after
        size_after = self.size_of(size_after)

        json_file = '/'.join(file_path.split('/')[-2:])

        template = "\033[34m{}\033[0m : " \
                   "\033[92mminimized\033[0m : " \
                   "\033[33m{}\033[0m -> \033[92m{}\033[0m".format(
            json_file,
            size_before,
            size_after,
        )

        print(template)

    def run(self):
        """Start json minimizer and exit when all json
        files was minimized.
        """
        for rel_path in sorted(self.paths):
            file_path = join(self.data_dir, rel_path)
            self.minify(file_path)

        after = self.size_of(self.after_total)
        before = self.size_of(self.before_total)
        saved = self.size_of(self.before_total - self.after_total)

        template = '\nTotal: ' \
                   '\033[92m{}\033[0m -> \033[92m{}\033[0m. ' \
                   'Compressed: \033[92m{}\033[0m\n'.format(before, after, saved)

        print(template)


class Versioner(BaseCommand):
    """Custom command for versioning"""

    def initialize_options(self):
        self.current = about['__version__']
        print('Previous version: '
              '\033[33m{}\033[0m.\n'.format(self.current))

    @staticmethod
    def automatically(version):
        """Automatically increment version string.

        :param version: Current version.
        :return: Next version.
        """
        major, minor, micro = [
            int(i) for i in version.split('.')
        ]

        if VERSION_MICRO_MAX > micro:
            micro += 1
        elif VERSION_MICRO_MAX == micro:
            micro = 0
            minor += 1
        elif VERSION_MINOR_MAX > minor:
            minor += 1
        elif VERSION_MINOR_MAX == minor:
            micro, minor = 0, 0
            major += 1
        if VERSION_MINOR_MAX < minor:
            minor, micro = 0, 0
            major += 1

        return '.'.join([str(i) for i
                         in (major, minor, micro)])

    def rewrite(self, version=None):
        if not version:
            version = self.current

        with open(join(here, 'mimesis', '__version__.py'), 'r+') as f:
            version_str = '__version__ = \'{}\''.format(version)
            regexp = r'__version__ = .*'

            meta = re.sub(regexp, version_str, f.read())
            f.seek(0)
            f.write(meta)
            f.truncate()

        print('Updated! Current version is: '
              '\033[34m{}\033[0m.\n'.format(version))

        exit()

    def run(self):
        response = input('Are you sure? (yes/no): ')
        if response.lower() in ('yes', 'y'):
            self.rewrite(
                self.automatically(
                    self.current,
                ),
            )


setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=__doc__,
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    license=about['__license__'],
    packages=[
        'mimesis',
        'mimesis.data',
        'mimesis.data.int',
        'mimesis.builtins',
        'mimesis.providers',
    ],
    keywords=[
        'fake',
        'mock',
        'data',
        'populate',
        'database',
        'testing',
        'generate',
        'mimesis',
        'dummy',
    ],
    package_data={
        'mimesis': [
            'data/*/*',
        ],
    },
    exclude_package_data={
        'mimesis': [
            # It's for development.
            'data/locale_template/*'
        ],
    },
    data_files=[
        ('', ['LICENSE']),
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Topic :: Software Development :: Testing',
    ],
    tests_require=tests_requirements,
    cmdclass={
        'test': PyTest,
        'versioner': Versioner,
        'minify': Minimizer,
        'upload': Upload,
    },
)



from mimesis.builtins.base import BaseSpecProvider


class JapanSpecProvider(BaseSpecProvider):
    """Class that provides special data for ja"""

    class Meta:
        name = 'japan_provider'

    @staticmethod
    def full_to_half(text: str, alnum: bool = True) -> str:
        """Convert all full width katakana, alphanumeric and few special
        characters like （, ）, ・ to equivalent half width character.

        :param str text: The text to be converted.
        :param bool alnum: Convert alphanumeric, default True.
        :return: Text with full width characters converted to half width.
        :rtype: str

        :Example:
            QVCｼﾞｬﾊﾟﾝ(0123)
        """
        fh_kana_special = {
            'ア': 'ｱ', 'イ': 'ｲ', 'ウ': 'ｳ', 'エ': 'ｴ', 'オ': 'ｵ', 'カ': 'ｶ',
            'キ': 'ｷ', 'ク': 'ｸ', 'ケ': 'ｹ', 'コ': 'ｺ', 'ガ': 'ｶﾞ', 'ギ': 'ｷﾞ',
            'グ': 'ｸﾞ', 'ゲ': 'ｹﾞ', 'ゴ': 'ｺﾞ', 'サ': 'ｻ', 'シ': 'ｼ', 'ス': 'ｽ',
            'セ': 'ｾ', 'ソ': 'ｿ', 'ザ': 'ｻﾞ', 'ジ': 'ｼﾞ', 'ズ': 'ｽﾞ', 'ゼ': 'ｾﾞ',
            'ゾ': 'ｿﾞ', 'タ': 'ﾀ', 'チ': 'ﾁ', 'ツ': 'ﾂ', 'テ': 'ﾃ', 'ト': 'ﾄ',
            'ダ': 'ﾀﾞ', 'ヂ': 'ﾁﾞ', 'ヅ': 'ﾂﾞ', 'デ': 'ﾃﾞ', 'ド': 'ﾄﾞ', 'ナ': 'ﾅ',
            'ニ': 'ﾆ', 'ヌ': 'ﾇ', 'ネ': 'ﾈ', 'ノ': 'ﾉ', 'ハ': 'ﾊ', 'ヒ': 'ﾋ',
            'フ': 'ﾌ', 'ヘ': 'ﾍ', 'ホ': 'ﾎ', 'バ': 'ﾊﾞ', 'ビ': 'ﾋﾞ', 'ブ': 'ﾌﾞ',
            'ベ': 'ﾍﾞ', 'ボ': 'ﾎﾞ', 'パ': 'ﾊﾟ', 'ピ': 'ﾋﾟ', 'プ': 'ﾌﾟ', 'ペ': 'ﾍﾟ',
            'ポ': 'ﾎﾟ', 'マ': 'ﾏ', 'ミ': 'ﾐ', 'ム': 'ﾑ', 'メ': 'ﾒ', 'モ': 'ﾓ',
            'ヤ': 'ﾔ', 'ユ': 'ﾕ', 'ヨ': 'ﾖ', 'ラ': 'ﾗ', 'リ': 'ﾘ', 'ル': 'ﾙ',
            'レ': 'ﾚ', 'ロ': 'ﾛ', 'ワ': 'ﾜ', 'ヲ': 'ｦ', 'ン': 'ﾝ', 'ァ': 'ｧ',
            'ィ': 'ｨ', 'ゥ': 'ｩ', 'ェ': 'ｪ', 'ォ': 'ｫ', 'ッ': 'ｯ', 'ャ': 'ｬ',
            'ュ': 'ｭ', 'ョ': 'ｮ', '！': '!', '＂': '\"', '＃': '#', '＄': '$',
            '％': '%', '＆': '&', '＇': '\'', '（': '(', '）': ')', '＊': '*',
            '＋': '+', 'ー': 'ｰ', '／': '/', '：': ':', '；': ';',
            '＜': '<', '＝': '=', '＞': '>', '？': '?', '＠': '@', '［': '[',
            '＼': '\\', '］': ']', '＾': '^', '＿': '_', '｀': '`', '｛': '{',
            '｜': '|', '｝': '}', '～': '~', '・': '･', '「': '｢', '」': '｣',
        }
        # leaving "。": "｡", "、": "," out for now

        _fh_alnum_offset = 65248  # 0xFEE0
        result = ''
        for char in text:
            if char in fh_kana_special:
                result += fh_kana_special[char]
            elif alnum and ord('０') <= ord(char) <= ord('ｚ'):
                result += chr(ord(char) - _fh_alnum_offset)
            else:
                result += char
        return result

    @staticmethod
    def half_to_full(text: str, alnum: bool = True) -> str:
        """Convert all half width katakana, alphanumeric, and special characters
        ((, ), ) to equivalent full width character.

        :param str text: The text to be converted.
        :param bool alnum: Convert alphanumeric, default True.
        :return: Text with half width characters converted to full width.
        :type: str

        :Example:
            ＱＶＣジャパン（０１２３）
        """
        _hf_alnum_offset = 65248
        result = ''

        hf_voiced_kana = {
            'ｶﾞ': 'ガ', 'ｷﾞ': 'ギ', 'ｸﾞ': 'グ', 'ｹﾞ': 'ゲ', 'ｺﾞ': 'ゴ', 'ｻﾞ': 'ザ',
            'ｼﾞ': 'ジ', 'ｽﾞ': 'ズ', 'ｾﾞ': 'ゼ', 'ｿﾞ': 'ゾ', 'ﾀﾞ': 'ダ', 'ﾁﾞ': 'ヂ',
            'ﾂﾞ': 'ヅ', 'ﾃﾞ': 'デ', 'ﾄﾞ': 'ド', 'ﾊﾞ': 'バ', 'ﾋﾞ': 'ビ', 'ﾌﾞ': 'ブ',
            'ﾍﾞ': 'ベ', 'ﾎﾞ': 'ボ', 'ﾊﾟ': 'パ', 'ﾋﾟ': 'ピ', 'ﾌﾟ': 'プ', 'ﾍﾟ': 'ペ',
            'ﾎﾟ': 'ポ',
        }
        hf_kana_special = {
            'ｱ': 'ア', 'ｲ': 'イ', 'ｳ': 'ウ', 'ｴ': 'エ', 'ｵ': 'オ', 'ｶ': 'カ',
            'ｷ': 'キ', 'ｸ': 'ク', 'ｹ': 'ケ', 'ｺ': 'コ', 'ｻ': 'サ', 'ｼ': 'シ',
            'ｽ': 'ス', 'ｾ': 'セ', 'ｿ': 'ソ', 'ﾀ': 'タ', 'ﾁ': 'チ', 'ﾂ': 'ツ',
            'ﾃ': 'テ', 'ﾄ': 'ト', 'ﾅ': 'ナ', 'ﾆ': 'ニ', 'ﾇ': 'ヌ', 'ﾈ': 'ネ',
            'ﾉ': 'ノ', 'ﾊ': 'ハ', 'ﾋ': 'ヒ', 'ﾌ': 'フ', 'ﾍ': 'ヘ', 'ﾎ': 'ホ',
            'ﾏ': 'マ', 'ﾐ': 'ミ', 'ﾑ': 'ム', 'ﾒ': 'メ', 'ﾓ': 'モ', 'ﾔ': 'ヤ',
            'ﾕ': 'ユ', 'ﾖ': 'ヨ', 'ﾗ': 'ラ', 'ﾘ': 'リ', 'ﾙ': 'ル', 'ﾚ': 'レ',
            'ﾛ': 'ロ', 'ﾜ': 'ワ', 'ｦ': 'ヲ', 'ﾝ': 'ン', 'ｧ': 'ァ', 'ｨ': 'ィ',
            'ｩ': 'ゥ', 'ｪ': 'ェ', 'ｫ': 'ォ', 'ｯ': 'ッ', 'ｬ': 'ャ', 'ｭ': 'ュ',
            'ｮ': 'ョ', '!': '！', '\"': '＂', '#': '＃', '$': '＄', '%': '％',
            '&': '＆', '\'': '＇', '(': '（', ')': '）', '*': '＊', '+': '＋',
            'ｰ': 'ー', '/': '／', ':': '：', ';': '；', '<': '＜', '=': '＝',
            '>': '＞', '?': '？', '@': '＠', '[': '［', '\\': '＼', ']': '］',
            '^': '＾', '_': '＿', '`': '｀', '{': '｛', '|': '｜', '}': '｝',
            '~': '～', '･': '・', '｢': '「', '｣': '」',
        }

        def hf_parse(char, result):
            """Parse the char from half-width to full-width, append to result,
            and return result.

            :param char: Character to be parsed from half-width to full-width.
            :param result: Previous result string.
            :return: Result appended with parsed char.
            :Example:
                ラーメン
            """
            if char in hf_kana_special:
                result += hf_kana_special[char]
            elif alnum and ord('0') <= ord(char) <= ord('z'):
                result += chr(ord(char) + _hf_alnum_offset)
            else:
                result += char
            return result

        # leave "｡": "。", ",": "、", for now
        i = 0
        while i < len(text) - 1:
            # need to lookahead for separate voice mark
            pair = text[i] + text[i + 1]
            if (text[i + 1] == 'ﾞ' or text[i + 1] == 'ﾟ') and \
                    pair in hf_voiced_kana:
                result += hf_voiced_kana[pair]
                i += 2
                continue
            else:
                result = hf_parse(text[i], result)
            i += 1

        if i == len(text) - 1:
            result = hf_parse(text[i], result)

        return result



from types import LambdaType
from typing import Any, Callable, List, Optional

from mimesis.exceptions import (UndefinedField, UndefinedSchema,
                                UnsupportedField)
from mimesis.providers.base import StrMixin
from mimesis.providers.generic import Generic
from mimesis.typing import JSON

__all__ = ['AbstractField', 'Field', 'Schema']


class AbstractField(StrMixin):
    """
    AbstractField is a class for generating data by the name of the method.

    Instance of this object takes any string which represents name
    of any method of any supported data provider (class ``Generic()``)
    and the ``**kwargs`` of the method:

    >>> _ = AbstractField('en')
    >>> _('full_name')
    'Benedict Larson'
    """

    def __init__(self, locale: str = 'en', seed: bool = None) -> None:
        self.locale = locale
        self.seed = seed
        self.gen = Generic(self.locale, self.seed)

    def __call__(self, name: Optional[str] = None,
                 key: Optional[Callable] = None, **kwargs) -> Any:
        """This magic override standard call so it's take any string which
        represents name of the any method of any supported data provider
        and the ``**kwargs`` of this method.

        :param name: Name of method.
        :param key: A key function (or other callable object)
            which will be applied to result.
        :param kwargs: Kwargs of method.
        :return: Value which represented by method.
        :raises ValueError: if provider is not
            supported or if field is not defined.
        """
        if name is None:
            raise UndefinedField()

        # TODO: This is a really slow solution. Fix it.
        for provider in dir(self.gen):
            if hasattr(self.gen, provider):
                provider = getattr(self.gen, provider)
                if name in dir(provider):
                    method = getattr(provider, name)
                    result = method(**kwargs)
                    if key and callable(key):
                        return key(result)
                    return result
            else:
                continue
        else:
            raise UnsupportedField(name)


class Schema(object):
    """
    Class which return list of filled schemas.
    """

    def __init__(self, schema: LambdaType) -> None:
        if callable(schema) and isinstance(schema, LambdaType):
            self.schema = schema
        else:
            raise UndefinedSchema()

    def create(self, iterations: int = 1) -> List[JSON]:
        data = map(lambda _: self.schema(), range(iterations))
        return list(data)


Field = AbstractField
